@@ -12,9 +12,11 @@
 #include "covermanager.h"
 #include "enginecontroller.h"
 #include "k3bexporter.h"
+#include "mediumpluginmanager.h"
 #include "playlistwindow.h"
 #include "playlist.h"
 #include "socketserver.h"       //Vis::Selector::showInstance()
+#include "threadweaver.h"
 
 #include <qpixmap.h>
 #include <qtooltip.h>
@@ -28,11 +30,26 @@
 #include <ktoolbarbutton.h>
 #include <kurl.h>
 
-using namespace amaroK;
+namespace Amarok
+{
+    bool repeatNone() { return AmarokConfig::repeat() == AmarokConfig::EnumRepeat::Off; }
+    bool repeatTrack() { return AmarokConfig::repeat() == AmarokConfig::EnumRepeat::Track; }
+    bool repeatAlbum() { return AmarokConfig::repeat() == AmarokConfig::EnumRepeat::Album; }
+    bool repeatPlaylist() { return AmarokConfig::repeat() == AmarokConfig::EnumRepeat::Playlist; }
+    bool randomOff() { return AmarokConfig::randomMode() == AmarokConfig::EnumRandomMode::Off; }
+    bool randomTracks() { return AmarokConfig::randomMode() == AmarokConfig::EnumRandomMode::Tracks; }
+    bool randomAlbums() { return AmarokConfig::randomMode() == AmarokConfig::EnumRandomMode::Albums; }
+    bool favorNone() { return AmarokConfig::favorTracks() == AmarokConfig::EnumFavorTracks::Off; }
+    bool favorScores() { return AmarokConfig::favorTracks() == AmarokConfig::EnumFavorTracks::HigherScores; }
+    bool favorRatings() { return AmarokConfig::favorTracks() == AmarokConfig::EnumFavorTracks::HigherRatings; }
+    bool favorLastPlay() { return AmarokConfig::favorTracks() == AmarokConfig::EnumFavorTracks::LessRecentlyPlayed; }
+    bool entireAlbums() { return repeatAlbum() || randomAlbums(); }
+}
+
+using namespace Amarok;
 
 KHelpMenu *Menu::s_helpMenu = 0;
 
-
 static void
 safePlug( KActionCollection *ac, const char *name, QWidget *w )
 {
@@ -50,7 +67,7 @@ safePlug( KActionCollection *ac, const char *name, QWidget *w )
 //////////////////////////////////////////////////////////////////////////////////////////
 
 MenuAction::MenuAction( KActionCollection *ac )
-  : KAction( i18n( "amaroK Menu" ), 0, ac, "amarok_menu" )
+  : KAction( i18n( "Amarok Menu" ), 0, ac, "amarok_menu" )
 {
     setShortcutConfigurable ( false ); //FIXME disabled as it doesn't work, should use QCursor::pos()
 }
@@ -74,7 +91,7 @@ MenuAction::plug( QWidget *w, int index )
         bar->alignItemRight( id );
 
         KToolBarButton* button = bar->getButton( id );
-        button->setPopup( amaroK::Menu::instance() );
+        button->setPopup( Amarok::Menu::instance() );
         button->setName( "toolbutton_amarok_menu" );
         button->setIcon( "amarok" );
 
@@ -85,27 +102,32 @@ MenuAction::plug( QWidget *w, int index )
 
 Menu::Menu()
 {
-    KActionCollection *ac = amaroK::actionCollection();
+    KActionCollection *ac = Amarok::actionCollection();
 
     setCheckable( true );
 
-    safePlug( ac, "repeat_track", this );
-    safePlug( ac, "repeat_playlist", this );
+    safePlug( ac, "repeat", this );
     safePlug( ac, "random_mode", this );
-    safePlug( ac, "dynamic_mode", this );
 
     insertSeparator();
 
-    insertItem( SmallIconSet( "covermanager" ), i18n( "C&over Manager" ), ID_SHOW_COVER_MANAGER );
+    safePlug( ac, "playlist_playmedia", this );
+    safePlug( ac, "play_audiocd", this );
+    safePlug( ac, "lastfm_play", this );
+
+    insertSeparator();
+
+    insertItem( SmallIconSet( Amarok::icon( "covermanager" ) ), i18n( "C&over Manager" ), ID_SHOW_COVER_MANAGER );
     safePlug( ac, "queue_manager", this );
-    insertItem( SmallIconSet( "visualizations"), i18n( "&Visualizations" ), ID_SHOW_VIS_SELECTOR );
-    insertItem( SmallIconSet( "equalizer"), i18n( "E&qualizer" ), kapp, SLOT( slotConfigEqualizer() ), 0, ID_CONFIGURE_EQUALIZER );
+    insertItem( SmallIconSet( Amarok::icon( "visualizations" ) ), i18n( "&Visualizations" ), ID_SHOW_VIS_SELECTOR );
+    insertItem( SmallIconSet( Amarok::icon( "equalizer" ) ), i18n( "E&qualizer" ), kapp, SLOT( slotConfigEqualizer() ), 0, ID_CONFIGURE_EQUALIZER );
     safePlug( ac, "script_manager", this );
+    safePlug( ac, "statistics", this );
 
     insertSeparator();
 
-    insertItem( SmallIconSet( "wizard" ), i18n( "First-Run &Wizard" ), ID_SHOW_WIZARD );
-    insertItem( i18n("&Rescan Collection"), ID_RESCAN_COLLECTION );
+    insertItem( SmallIconSet( Amarok::icon( "rescan" ) ), i18n("&Rescan Collection"), ID_RESCAN_COLLECTION );
+    setItemEnabled( ID_RESCAN_COLLECTION, !ThreadWeaver::instance()->isJobPending( "CollectionScanner" ) );
 
     insertSeparator();
 
@@ -151,10 +173,10 @@ Menu::helpMenu( QWidget *parent ) //STATIC
     extern KAboutData aboutData;
 
     if ( s_helpMenu == 0 )
-        s_helpMenu = new KHelpMenu( parent, &aboutData, amaroK::actionCollection() );
+        s_helpMenu = new KHelpMenu( parent, &aboutData, Amarok::actionCollection() );
     return s_helpMenu->menu();
 
-    return (new KHelpMenu( parent, &aboutData, amaroK::actionCollection() ))->menu();
+    return (new KHelpMenu( parent, &aboutData, Amarok::actionCollection() ))->menu();
 }
 
 void
@@ -172,11 +194,6 @@ Menu::slotActivated( int index )
     case ID_SHOW_COVER_MANAGER:
         CoverManager::showOnce();
         break;
-    case ID_SHOW_WIZARD:
-        pApp->firstRunWizard();
-        pApp->playlistWindow()->recreateGUI();
-        pApp->applySettings();
-        break;
     case ID_SHOW_VIS_SELECTOR:
         Vis::Selector::instance()->show(); //doing it here means we delay creation of the widget
         break;
@@ -207,17 +224,17 @@ PlayPauseAction::engineStateChanged( Engine::State state,  Engine::State /*oldSt
     switch( state ) {
     case Engine::Playing:
         setChecked( false );
-        setIcon( "player_pause" );
+        setIcon( Amarok::icon( "pause" ) );
         text = i18n( "Pause" );
         break;
     case Engine::Paused:
         setChecked( true );
-        setIcon( "player_pause" );
+        setIcon( Amarok::icon( "pause" ) );
         text = i18n( "Pause" );
         break;
     case Engine::Empty:
         setChecked( false );
-        setIcon( "player_play" );
+        setIcon( Amarok::icon( "play" ) );
         text = i18n( "Play" );
         break;
     case Engine::Idle:
@@ -304,7 +321,7 @@ AnalyzerContainer::mousePressEvent( QMouseEvent *e)
     else if( e->button() == Qt::RightButton ) {
         #if defined HAVE_XMMS || defined HAVE_LIBVISUAL
         KPopupMenu menu;
-        menu.insertItem( SmallIconSet( "visualizations" ), i18n("&Visualizations"), Menu::ID_SHOW_VIS_SELECTOR );
+        menu.insertItem( SmallIconSet( Amarok::icon( "visualizations" ) ), i18n("&Visualizations"), Menu::ID_SHOW_VIS_SELECTOR );
 
         if( menu.exec( mapToGlobal( e->pos() ) ) == Menu::ID_SHOW_VIS_SELECTOR )
             Menu::instance()->slotActivated( Menu::ID_SHOW_VIS_SELECTOR );
@@ -312,6 +329,82 @@ AnalyzerContainer::mousePressEvent( QMouseEvent *e)
     }
 }
 
+//////////////////////////////////////////////////////////////////////////////////////////
+// ToggleAction
+//////////////////////////////////////////////////////////////////////////////////////////
+
+ToggleAction::ToggleAction( const QString &text, void ( *f ) ( bool ), KActionCollection* const ac, const char *name )
+        : KToggleAction( text, 0, ac, name )
+        , m_function( f )
+{}
+
+void ToggleAction::setChecked( bool b )
+{
+    const bool announce = b != isChecked();
+
+    m_function( b );
+    KToggleAction::setChecked( b );
+    AmarokConfig::writeConfig(); //So we don't lose the setting when crashing
+    if( announce ) emit toggled( b ); //KToggleAction doesn't do this for us. How gay!
+}
+
+void ToggleAction::setEnabled( bool b )
+{
+    const bool announce = b != isEnabled();
+
+    if( !b )
+        setChecked( false );
+    KToggleAction::setEnabled( b );
+    AmarokConfig::writeConfig(); //So we don't lose the setting when crashing
+    if( announce ) emit enabled( b );
+}
+
+//////////////////////////////////////////////////////////////////////////////////////////
+// SelectAction
+//////////////////////////////////////////////////////////////////////////////////////////
+
+SelectAction::SelectAction( const QString &text, void ( *f ) ( int ), KActionCollection* const ac, const char *name )
+        : KSelectAction( text, 0, ac, name )
+        , m_function( f )
+{ }
+
+void SelectAction::setCurrentItem( int n )
+{
+    const bool announce = n != currentItem();
+
+    m_function( n );
+    KSelectAction::setCurrentItem( n );
+    AmarokConfig::writeConfig(); //So we don't lose the setting when crashing
+    if( announce ) emit activated( n );
+}
+
+void SelectAction::setEnabled( bool b )
+{
+    const bool announce = b != isEnabled();
+
+    if( !b )
+        setCurrentItem( 0 );
+    KSelectAction::setEnabled( b );
+    AmarokConfig::writeConfig(); //So we don't lose the setting when crashing
+    if( announce ) emit enabled( b );
+}
+
+void SelectAction::setIcons( QStringList icons )
+{
+    m_icons = icons;
+    for( int i = 0, n = items().count(); i < n; ++i )
+        popupMenu()->changeItem( i, kapp->iconLoader()->loadIconSet( *icons.at( i ), KIcon::Small ), popupMenu()->text( i ) );
+}
+
+QStringList SelectAction::icons() const { return m_icons; }
+
+QString SelectAction::currentIcon() const
+{
+    if( m_icons.count() )
+        return *m_icons.at( currentItem() );
+    return QString::null;
+}
+
 
 //////////////////////////////////////////////////////////////////////////////////////////
 // VolumeAction
@@ -328,9 +421,9 @@ VolumeAction::plug( QWidget *w, int index )
 {
     //NOTE we only support one plugging currently
 
-    delete (amaroK::VolumeSlider*) m_slider; //just in case, remember, we only support one plugging!
+    delete static_cast<Amarok::VolumeSlider*>( m_slider ); //just in case, remember, we only support one plugging!
 
-    m_slider = new amaroK::VolumeSlider( w, amaroK::VOLUME_MAX );
+    m_slider = new Amarok::VolumeSlider( w, Amarok::VOLUME_MAX );
     m_slider->setName( "ToolBarVolume" );
     m_slider->setValue( AmarokConfig::masterVolume() );
     m_slider->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Ignored );
@@ -358,40 +451,47 @@ VolumeAction::engineVolumeChanged( int value )
 // RandomAction
 //////////////////////////////////////////////////////////////////////////////////////////
 RandomAction::RandomAction( KActionCollection *ac ) :
-    ToggleAction( i18n( "Random &Mode" ), &AmarokConfig::setRandomMode, ac, "random_mode" )
+    SelectAction( i18n( "Ra&ndom" ), &AmarokConfig::setRandomMode, ac, "random_mode" )
 {
-    KToggleAction::setChecked( AmarokConfig::randomMode() );
-    setIcon( "random" );
+    setItems( QStringList() << i18n( "&Off" ) << i18n( "&Tracks" ) << i18n( "&Albums" ) );
+    setCurrentItem( AmarokConfig::randomMode() );
+    setIcons( QStringList() << Amarok::icon( "random_no" ) << Amarok::icon( "random_track" ) << Amarok::icon( "random_album" ) );
+}
+
+void
+RandomAction::setCurrentItem( int n )
+{
+    if( KAction *a = parentCollection()->action( "favor_tracks" ) )
+        a->setEnabled( n );
+    SelectAction::setCurrentItem( n );
+}
+
+
+//////////////////////////////////////////////////////////////////////////////////////////
+// FavorAction
+//////////////////////////////////////////////////////////////////////////////////////////
+FavorAction::FavorAction( KActionCollection *ac ) :
+    SelectAction( i18n( "&Favor" ), &AmarokConfig::setFavorTracks, ac, "favor_tracks" )
+{
+    setItems( QStringList() << i18n( "Off" )
+                            << i18n( "Higher &Scores" )
+                            << i18n( "Higher &Ratings" )
+                            << i18n( "Not Recently &Played" ) );
+
+    setCurrentItem( AmarokConfig::favorTracks() );
+    setEnabled( AmarokConfig::randomMode() );
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
-// RepeatTrackAction
+// RepeatAction
 //////////////////////////////////////////////////////////////////////////////////////////
-RepeatTrackAction::RepeatTrackAction( KActionCollection *ac ) :
-    ToggleAction( i18n( "Repeat &Track" ), &AmarokConfig::setRepeatTrack, ac, "repeat_track" )
+RepeatAction::RepeatAction( KActionCollection *ac ) :
+    SelectAction( i18n( "&Repeat" ), &AmarokConfig::setRepeat, ac, "repeat" )
 {
-    KToggleAction::setChecked( AmarokConfig::repeatTrack() );
-    setIcon( "repeat_track" );
-}
-
-//////////////////////////////////////////////////////////////////////////////////////////
-// RepeatPlaylistAction
-//////////////////////////////////////////////////////////////////////////////////////////
-RepeatPlaylistAction::RepeatPlaylistAction( KActionCollection *ac ) :
-    ToggleAction( i18n( "R&epeat Playlist" ), &AmarokConfig::setRepeatPlaylist, ac, "repeat_playlist" )
-{
-    KToggleAction::setChecked( AmarokConfig::repeatPlaylist() );
-    setIcon( "repeat_playlist" );
-}
-
-//////////////////////////////////////////////////////////////////////////////////////////
-// PartyAction
-//////////////////////////////////////////////////////////////////////////////////////////
-DynamicAction::DynamicAction( KActionCollection *ac ) :
-    ToggleAction( i18n( "&Dynamic Mode" ), &AmarokConfig::setDynamicMode, ac, "dynamic_mode" )
-{
-    KToggleAction::setChecked( AmarokConfig::dynamicMode() );
-    setIcon( "dynamic" );
+    setItems( QStringList() << i18n( "&Off" ) << i18n( "&Track" )
+                            << i18n( "&Album" ) << i18n( "&Playlist" ) );
+    setIcons( QStringList() << Amarok::icon( "repeat_no" ) << Amarok::icon( "repeat_track" ) << Amarok::icon( "repeat_album" ) << Amarok::icon( "repeat_playlist" ) );
+    setCurrentItem( AmarokConfig::repeat() );
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -399,8 +499,7 @@ DynamicAction::DynamicAction( KActionCollection *ac ) :
 //////////////////////////////////////////////////////////////////////////////////////////
 BurnMenuAction::BurnMenuAction( KActionCollection *ac )
   : KAction( i18n( "Burn" ), 0, ac, "burn_menu" )
-{
-}
+{}
 
 int
 BurnMenuAction::plug( QWidget *w, int index )
@@ -417,7 +516,7 @@ BurnMenuAction::plug( QWidget *w, int index )
         bar->insertButton( QString::null, id, true, i18n( "Burn" ), index );
 
         KToolBarButton* button = bar->getButton( id );
-        button->setPopup( amaroK::BurnMenu::instance() );
+        button->setPopup( Amarok::BurnMenu::instance() );
         button->setName( "toolbutton_burn_menu" );
         button->setIcon( "k3b" );
 
@@ -445,9 +544,7 @@ BurnMenu::instance()
 
 void
 BurnMenu::slotAboutToShow()
-{
-
-}
+{}
 
 void
 BurnMenu::slotActivated( int index )
@@ -469,7 +566,7 @@ BurnMenu::slotActivated( int index )
 //////////////////////////////////////////////////////////////////////////////////////////
 
 StopAction::StopAction( KActionCollection *ac )
-  : KAction( i18n( "Stop" ), "player_stop", 0, EngineController::instance(), SLOT( stop() ), ac, "stop" )
+  : KAction( i18n( "Stop" ), Amarok::icon( "stop" ), 0, EngineController::instance(), SLOT( stop() ), ac, "stop" )
 {}
 
 int
@@ -488,10 +585,10 @@ StopAction::plug( QWidget *w, int index )
                            true, i18n( "Stop" ), index );
 
         KToolBarButton* button = bar->getButton( id );
-        button->setDelayedPopup( amaroK::StopMenu::instance() );
+        button->setDelayedPopup( Amarok::StopMenu::instance() );
         button->setName( "toolbutton_stop_menu" );
-        button->setIcon( "player_stop" );
-        button->setEnabled( false ); // Required, otherwise the button is always enabled at startup
+        button->setIcon( Amarok::icon( "stop" ) );
+        button->setEnabled( EngineController::instance()->engine()->loaded() );  // Disable button at startup
 
         return containerCount() - 1;
     }
@@ -521,12 +618,12 @@ StopMenu::slotAboutToShow()
 {
     Playlist *pl = Playlist::instance();
 
-    setItemEnabled( NOW,         amaroK::actionCollection()->action( "stop" )->isEnabled() );
+    setItemEnabled( NOW,         Amarok::actionCollection()->action( "stop" )->isEnabled() );
 
     setItemEnabled( AFTER_TRACK, pl->currentTrackIndex() >= 0 );
     setItemChecked( AFTER_TRACK, pl->stopAfterMode() == Playlist::StopAfterCurrent );
 
-    setItemEnabled( AFTER_QUEUE, pl->nextTracksCount() );
+    setItemEnabled( AFTER_QUEUE, pl->nextTracks().count() );
     setItemChecked( AFTER_QUEUE, pl->stopAfterMode() == Playlist::StopAfterQueue );
 }
 
@@ -539,7 +636,7 @@ StopMenu::slotActivated( int index )
     switch( index )
     {
         case NOW:
-            amaroK::actionCollection()->action( "stop" )->activate();
+            Amarok::actionCollection()->action( "stop" )->activate();
             if( mode == Playlist::StopAfterCurrent || mode == Playlist::StopAfterQueue )
                 pl->setStopAfterMode( Playlist::DoNotStop );
             break;
