@@ -12,7 +12,6 @@
 #include "amarokconfig.h"
 #include "browserbar.h"
 #include "debug.h"
-#include "enginecontroller.h"
 #include "multitabbar.h"   //m_tabBar
 
 #include <kapplication.h>  //kapp
@@ -23,14 +22,15 @@
 #include <qcursor.h>       //for resize cursor
 #include <qpainter.h>
 #include <qsignalmapper.h> //m_mapper
-#include <qstyle.h>        //amaroK::Splitter
+#include <qstyle.h>        //Amarok::Splitter
+#include <qtooltip.h>
 
 
 // we emulate a qsplitter, mostly for historic reasons, but there are still a few advantages
 // mostly we can stop the browserbar getting resized too small so that switching browser looks wrong
 
 
-namespace amaroK
+namespace Amarok
 {
     class Splitter : public QWidget {
     public:
@@ -58,25 +58,42 @@ namespace amaroK
     };
 }
 
+BrowserBar* BrowserBar::s_instance = 0;
 
 BrowserBar::BrowserBar( QWidget *parent )
         : QWidget( parent, "BrowserBar" )
-        , EngineObserver( EngineController::instance() )
         , m_playlistBox( new QVBox( this ) )
-        , m_divider( new amaroK::Splitter( this ) )
-        , m_tabBar( new MultiTabBar( MultiTabBar::Vertical, this ) )
-        , m_browserBox( new QWidget( this ) )
+        , m_divider( new Amarok::Splitter( this ) )
+        , m_browserBox( new QVBox( this ) )
         , m_currentIndex( -1 )
         , m_lastIndex( -1 )
         , m_mapper( new QSignalMapper( this ) )
 {
+
+ 
+
+    m_tabManagementButton = new QPushButton( SmallIconSet(Amarok::icon( "configure" )), 0, this, "tab_managment_button" );
+    connect (m_tabManagementButton, SIGNAL(clicked()), SLOT(showBrowserSelectionMenu()));
+    m_tabManagementButton->setIsMenuButton ( true ); //deprecated, but since I cannot add menu directly to button it is needed.
+
+    QToolTip::add (m_tabManagementButton, i18n("Manage tabs")); 
+
+
+    m_tabBar = new MultiTabBar( MultiTabBar::Vertical, this );
+
+
+    m_tabManagementButton->setFixedWidth(m_tabBar->sizeHint().width());
+    m_tabManagementButton->setFixedHeight(m_tabBar->sizeHint().width());
+
+
+    s_instance = this;
     m_pos = m_tabBar->sizeHint().width() + 5; //5 = aesthetic spacing
 
     m_tabBar->setStyle( MultiTabBar::AMAROK );
     m_tabBar->setPosition( MultiTabBar::Left );
     m_tabBar->showActiveTabTexts( true );
     m_tabBar->setFixedWidth( m_pos );
-    m_tabBar->move( 0, 3 );
+    m_tabBar->move( 0, 25 );
 
     QVBoxLayout *layout = new QVBoxLayout( m_browserBox );
     layout->addSpacing( 3 ); // aesthetics
@@ -88,15 +105,43 @@ BrowserBar::BrowserBar( QWidget *parent )
     m_playlistBox->setSpacing( 1 );
 
     connect( m_mapper, SIGNAL(mapped( int )), SLOT(showHideBrowser( int )) );
+
+ 
+
+
+   
+    //m_tabBar->appendButton( Amarok::icon( "configure" ), 1, 0, QString::null );
+
 }
 
 BrowserBar::~BrowserBar()
 {
-    KConfig* const config = amaroK::config( "BrowserBar" );
+    KConfig* const config = Amarok::config( "BrowserBar" );
     config->writeEntry( "CurrentPane", m_currentIndex != -1 ? QString(currentBrowser()->name()) : QString::null );
     config->writeEntry( "Width", m_browserBox->width() );
 }
 
+void
+BrowserBar::makeDropProxy( const QString &name, DropProxyTarget *finalTarget )
+{
+    int id = m_browserIds[name];
+    MultiTabBarButton *button = m_tabBar->tab( id );
+    if( button )
+        button->proxyDrops( finalTarget );
+}
+
+int
+BrowserBar::restoreWidth()
+{
+    const int index = indexForName( Amarok::config( "BrowserBar" )->readEntry( "CurrentPane" ) );
+    const int width = Amarok::config( "BrowserBar" )->readNumEntry( "Width", browser( index )->sizeHint().width() );
+
+    m_browserBox->resize( width, height() );
+    m_pos = m_browserBox->width() + m_tabBar->width();
+
+    return index;
+}
+
 void
 BrowserBar::polish()
 {
@@ -107,22 +152,18 @@ BrowserBar::polish()
     uint M = 0;
     foreachType( BrowserList, m_browsers ) {
         const uint m = (*it)->minimumWidth();
-        if( m > M ) M = m;
-    }
-
-    KConfig* const config = amaroK::config( "BrowserBar" );
-    const int index = indexForName( config->readEntry( "CurrentPane" ) );
-    const int width = config->readNumEntry( "Width", browser( index )->sizeHint().width() );
-
-    if( M > 250 ) {
-        warning() << "Some browsers are insisting on a silly minimum size! " << M << endl;
-        M = 250;
+        if (m > M)
+            M = m;
+        if (m > 250) {
+            warning() << "Browser is too large, mxcl says castrate the developer: " << (*it)->name() << ", " << M << endl;
+            M = 250;
+        }
     }
 
     m_browserBox->setMinimumWidth( M );
-    m_browserBox->resize( width, height() );
+    const int index = restoreWidth();
 
-    if( index != -1 )
+    if (index != -1)
         // if we did it for -1 it ruins the browserBox size
         showHideBrowser( index );
 }
@@ -133,12 +174,12 @@ BrowserBar::adjustWidgetSizes()
     //TODO set the geometry of the PlaylistWindow before
     // the browsers are loaded so this isn't called twice
 
-    const uint w   = width();
-    const uint h   = height();
-    const uint mxW = maxBrowserWidth();
-    const uint p   = (m_pos < mxW) ? m_pos : mxW;
-    const uint ppw = p + m_divider->width();
-    const uint tbw = m_tabBar->width();
+    const uint w    = width();
+    const uint h    = height();
+    const uint maxW = maxBrowserWidth();
+    const uint p    = (m_pos < maxW) ? m_pos : maxW;
+    const uint ppw  = p + m_divider->width();
+    const uint tbw  = m_tabBar->width();
 
     m_divider->move( p, 0 );
 
@@ -201,16 +242,17 @@ BrowserBar::event( QEvent *e )
 }
 
 void
-BrowserBar::addBrowser( QWidget *widget, const QString &title, const QString& icon )
+BrowserBar::addBrowser( const QString &identifier, QWidget *widget, const QString &title, const QString& icon )
 {
     const int id = m_tabBar->tabs()->count(); // the next available id
     const QString name( widget->name() );
+    m_browserIds[name] = id;
     QWidget *tab;
 
     widget->reparent( m_browserBox, QPoint() );
     widget->hide();
 
-    m_tabBar->appendTab( SmallIcon( icon ), id, title );
+    m_tabBar->appendTab( SmallIcon( icon ), id, title, identifier );
     tab = m_tabBar->tab( id );
     tab->setFocusPolicy( QWidget::NoFocus ); //FIXME you can focus on the tab, but they respond to no input!
 
@@ -222,6 +264,18 @@ BrowserBar::addBrowser( QWidget *widget, const QString &title, const QString& ic
     m_browsers.push_back( widget );
 }
 
+void
+BrowserBar::removeMediaBrowser( QWidget *widget )
+{
+    BrowserList::iterator it = qFind( m_browsers.begin(), m_browsers.end(), widget );
+    if( it != m_browsers.end() )
+        m_browsers.erase( it );
+    QWidget *tab;
+    tab = m_tabBar->tab( m_browserIds["MediaBrowser"] );
+    m_mapper->removeMappings( tab );
+    m_tabBar->removeTab( m_browserIds["MediaBrowser"] );
+}
+
 void
 BrowserBar::showHideBrowser( int index )
 {
@@ -264,6 +318,28 @@ BrowserBar::showHideBrowser( int index )
             adjustWidgetSizes();
         }
     }
+
+    emit browserActivated( index );
+}
+
+void
+BrowserBar::showHideVisibleBrowser( int index )
+{
+    int realindex = -1;
+    QPtrList<MultiTabBarTab> tabs = *m_tabBar->tabs();
+    for( int i = 0, n = tabs.count(); i < n; ++i )
+    {
+        if( tabs.at( i )->visible() )
+            index--;
+        if( index < 0 )
+        {
+            realindex = i;
+            break;
+        }
+    }
+
+    if( realindex >= 0 )
+        showHideBrowser( realindex );
 }
 
 QWidget*
@@ -276,88 +352,32 @@ BrowserBar::browser( const QString &name ) const
     return 0;
 }
 
+int
+BrowserBar::visibleCount() const
+{
+    int num = 0;
+    QPtrList<MultiTabBarTab> tabs = *m_tabBar->tabs();
+    for( int i = 0, n = tabs.count(); i < n; ++i )
+    {
+        if( tabs.at( i )->visible() )
+            num++;
+    }
+
+    return num;
+}
+
 int
 BrowserBar::indexForName( const QString &name ) const
 {
     for( uint x = 0; x < m_browsers.count(); ++x )
         if( name == m_browsers[x]->name() )
             return x;
-
     return -1;
 }
 
-void
-BrowserBar::engineStateChanged( Engine::State state, Engine::State oldState )
+void BrowserBar::showBrowserSelectionMenu()
 {
-    if( !AmarokConfig::autoShowContextBrowser() || m_currentIndex == -1 )
-        return;
-
-    switch( state ) {
-    case Engine::Playing:
-
-        if( oldState != Engine::Paused && m_currentIndex != -1 ) {
-            m_lastIndex = m_currentIndex;
-            showBrowser( "ContextBrowser" );
-        }
-
-        // we watch for any event, if there's some event we reset the timer
-//         currentBrowser()->installEventFilter( this );
-//         currentBrowser()->setMouseTracking( true );
-//         killTimers();
-//         startTimer( 5000 );
-        break;
-
-    case Engine::Empty:
-
-        if( m_lastIndex >= 0 )
-            showBrowser( m_lastIndex );
-
-    default:
-        ;
-    }
-}
-
-bool
-BrowserBar::eventFilter( QObject *o, QEvent *e )
-{
-    DEBUG_FUNC_INFO;
-
-    switch( e->type() )
-    {
-    case QEvent::MouseMove:
-    case QEvent::MouseButtonRelease:
-    case QEvent::MouseButtonPress:
-    case QEvent::FocusIn:
-    case QEvent::KeyPress:
-    case QEvent::KeyRelease:
-
-        // we put an event filter on this browser to check
-        // if it is still being used within 5 seconds of
-        // playback starting. If so we shouldn't auto-switch
-        // to the context browser
-        o->removeEventFilter( this );
-        killTimers();
-        break;
-
-    default:
-        ;
-    }
-
-    return false;
-}
-
-void
-BrowserBar::timerEvent( QTimerEvent* )
-{
-    if( m_currentIndex != -1 ) //means the browsers are closed
-        showBrowser( "ContextBrowser" );
-
-    // it might be bad to leave excess filters running
-    // so just in case
-    foreachType( BrowserList, m_browsers )
-        (*it)->removeEventFilter( this );
-
-    killTimers();
+      m_tabBar->showTabSelectionMenu(mapToGlobal(QPoint(m_tabManagementButton->pos().x(), m_tabManagementButton->pos().y() +m_tabManagementButton->height() )));
 }
 
 #include "browserbar.moc"
