@@ -25,10 +25,12 @@ class QComboBox;
 class QGroupBox;
 class QVBox;
 
-namespace amaroK {
+namespace Amarok {
     class PluginConfig;
 }
 
+class MediumPluginManager;
+
 class AmarokConfigDialog : public KConfigDialog
 {
     Q_OBJECT
@@ -43,27 +45,30 @@ class AmarokConfigDialog : public KConfigDialog
         void showPage( const QCString& page );
 
     protected slots:
+        void updateButtons();
         void updateSettings();
         void updateWidgets();
         void updateWidgetsDefault();
 
     private slots:
         void aboutEngine();
+
     protected:
         bool hasChanged();
         bool isDefault();
 
     private:
         void soundSystemChanged();
-        const QString databaseType();
+        QString externalBrowser() const;
 
         QComboBox* m_soundSystem;
-        amaroK::PluginConfig *m_engineConfig;
+        Amarok::PluginConfig *m_engineConfig;
         QGroupBox            *m_engineConfigFrame;
         class Options1       *m_opt1;
         class Options2       *m_opt2;
         class Options4       *m_opt4;
         class Options7       *m_opt7;
+        MediumPluginManager  *m_deviceManager;
 
         QValueList<QWidget*> m_pageList;
         QMap<QString, QString> m_pluginName;
