@@ -1,4 +1,5 @@
 // (c) Pierpaolo Di Panfilo 2004
+// (c) 2005 Isaiah Damron <xepo@trifault.net>
 // See COPYING file for licensing information
 
 #include "amarok.h"
@@ -6,6 +7,7 @@
 #include "browserToolBar.h"
 #include "clicklineedit.h"
 #include "debug.h"
+#include "collectionbrowser.h" //manipulateThe()
 #include "collectiondb.h"
 #include "config.h"
 #include "coverfetcher.h"
@@ -47,16 +49,13 @@
 #include <ksqueezedtextlabel.h> //status label
 #include <kstandarddirs.h>   //KGlobal::dirs()
 #include <kstatusbar.h>
+#include <kstringhandler.h>    //paintItem
 #include <ktoolbar.h>
 #include <ktoolbarbutton.h>    //clear filter button
 #include <kurl.h>
 #include <kurldrag.h>
 #include <kwin.h>
 
-
-//Tooltips
-#define escapeHTMLAttr(s) QString(s).replace( "%", "%25" ).replace( "'", "%27" ).replace( "#", "%23" ).replace( "?", "%3F" )
-
 static QString artistToSelectInInitFunction;
 CoverManager *CoverManager::s_instance = 0;
 
@@ -83,23 +82,35 @@ CoverManager::CoverManager()
     m_artistView = new KListView( this );
     m_artistView->addColumn(i18n( "Albums By" ));
     m_artistView->setFullWidth( true );
-    m_artistView->setSorting( -1 );    //no sort
+    m_artistView->setSorting( 0 );
     m_artistView->setMinimumWidth( 180 );
-    KListViewItem *item = new KListViewItem( m_artistView, i18n( "All Albums" ) );
-    item->setPixmap( 0, SmallIcon("cdrom_unmount") );
+    KListViewItem *item = 0;
 
     //load artists from the collection db
     const QStringList artists = CollectionDB::instance()->artistList( false, false );
-    foreach( artists )  {
-        item = new KListViewItem( m_artistView, item, *it );
-        item->setPixmap( 0, SmallIcon("personal") );
+    foreach( artists )
+    {
+        QString artist = *it;
+        if( artist.startsWith( "The " ) )
+            CollectionView::instance()->manipulateThe( artist, true );
+
+        item = new KListViewItem( m_artistView, item, artist );
+        item->setPixmap( 0, SmallIcon( Amarok::icon( "artist" ) ) );
     }
+    m_artistView->sort();
+
+    m_artistView->setSorting( -1 );
+    KListViewItem *last = (KListViewItem *)m_artistView->lastItem();
+    item = new KListViewItem( m_artistView, 0, i18n( "All Albums" ) );
+    item->setPixmap( 0, SmallIcon( Amarok::icon( "album" ) ) );
+
     QueryBuilder qb;
     qb.addReturnValue( QueryBuilder::tabAlbum, QueryBuilder::valName );
-    qb.setOptions( QueryBuilder::optOnlyCompilations | QueryBuilder::optRemoveDuplicates );
+    qb.groupBy( QueryBuilder::tabAlbum, QueryBuilder::valName );
+    qb.setOptions( QueryBuilder::optOnlyCompilations );
     qb.setLimit( 0, 1 );
     if ( qb.run().count() ) {
-        item = new KListViewItem( m_artistView, item, i18n( "Various Artists" ) );
+        item = new KListViewItem( m_artistView, last, i18n( "Various Artists" ) );
         item->setPixmap( 0, SmallIcon("personal") );
     }
 
@@ -113,7 +124,7 @@ CoverManager::CoverManager()
         QHBox *searchBox = new QHBox( hbox );
         KToolBar* searchToolBar = new Browser::ToolBar( searchBox );
         KToolBarButton *button = new KToolBarButton( "locationbar_erase", 0, searchToolBar );
-        m_searchEdit = new ClickLineEdit( i18n( "Filter here..." ), searchToolBar );
+        m_searchEdit = new ClickLineEdit( i18n( "Enter search terms here" ), searchToolBar );
         m_searchEdit->setFrame( QFrame::Sunken );
 
         searchToolBar->setStretchableWidget( m_searchEdit );
@@ -136,12 +147,12 @@ CoverManager::CoverManager()
     #ifdef AMAZON_SUPPORT
     // amazon locale menu
     m_amazonLocaleMenu = new KPopupMenu( this );
-    m_amazonLocaleMenu->insertItem( i18n("International"), International );
-    m_amazonLocaleMenu->insertItem( i18n("Canada"), Canada );
-    m_amazonLocaleMenu->insertItem( i18n("France"), France );
-    m_amazonLocaleMenu->insertItem( i18n("Germany"), Germany );
-    m_amazonLocaleMenu->insertItem( i18n("Japan"), Japan);
-    m_amazonLocaleMenu->insertItem( i18n("United Kingdom"), UK );
+    m_amazonLocaleMenu->insertItem( i18n("International"), CoverFetcher::International );
+    m_amazonLocaleMenu->insertItem( i18n("Canada"), CoverFetcher::Canada );
+    m_amazonLocaleMenu->insertItem( i18n("France"), CoverFetcher::France );
+    m_amazonLocaleMenu->insertItem( i18n("Germany"), CoverFetcher::Germany );
+    m_amazonLocaleMenu->insertItem( i18n("Japan"), CoverFetcher::Japan);
+    m_amazonLocaleMenu->insertItem( i18n("United Kingdom"), CoverFetcher::UK );
     connect( m_amazonLocaleMenu, SIGNAL( activated(int) ), SLOT( changeLocale(int) ) );
     #endif
 
@@ -150,26 +161,14 @@ CoverManager::CoverManager()
     toolBar->setFrameShape( QFrame::NoFrame );
     toolBar->insertButton( "view_choose", 1, m_viewMenu, true, i18n( "View" ) );
     #ifdef AMAZON_SUPPORT
-    const QString babelfish = locate( "data", QString( "amarok/images/babelfish.png" ) );
-    toolBar->insertButton( babelfish, 2, m_amazonLocaleMenu, true, i18n( "Amazon Locale" ) );
+    toolBar->insertButton( "babelfish", 2, m_amazonLocaleMenu, true, i18n( "Amazon Locale" ) );
 
     QString locale = AmarokConfig::amazonLocale();
-
-         if( locale == "fr" ) m_currentLocale = France;
-    else if( locale == "de" ) m_currentLocale = Germany;
-    else if( locale == "jp" ) m_currentLocale = Japan;
-    else if( locale == "uk" ) m_currentLocale = UK;
-    else if( locale == "ca" ) m_currentLocale = Canada;
-    else {
-        // make sure we handle old config files correctly
-        locale = "us";
-        m_currentLocale = International;
-    }
-
+    m_currentLocale = CoverFetcher::localeStringToID( locale );
     m_amazonLocaleMenu->setItemChecked( m_currentLocale, true );
 
     //fetch missing covers button
-    m_fetchButton = new KPushButton( KGuiItem( i18n("Fetch Missing Covers"), "cdrom_unmount" ), hbox );
+    m_fetchButton = new KPushButton( KGuiItem( i18n("Fetch Missing Covers"), Amarok::icon( "download" ) ), hbox );
     connect( m_fetchButton, SIGNAL(clicked()), SLOT(fetchMissingCovers()) );
     #endif
 
@@ -216,7 +215,7 @@ CoverManager::CoverManager()
     m_currentView = AllAlbums;
 
     QSize size = QApplication::desktop()->screenGeometry( this ).size() / 1.5;
-    resize( amaroK::config( "Cover Manager" )->readSizeEntry( "Window Size", &size ) );
+    resize( Amarok::config( "Cover Manager" )->readSizeEntry( "Window Size", &size ) );
 
     show();
 
@@ -228,7 +227,7 @@ CoverManager::~CoverManager()
 {
     DEBUG_BLOCK
 
-    amaroK::config( "Cover Manager" )->writeEntry( "Window Size", size() );
+    Amarok::config( "Cover Manager" )->writeEntry( "Window Size", size() );
 
     s_instance = 0;
 }
@@ -252,16 +251,16 @@ void CoverManager::init()
 }
 
 
-CoverViewDialog::CoverViewDialog(const QString& artist, const QString& album, QWidget *parent)
-    : QDialog(parent, 0, false, WDestructiveClose | WType_TopLevel | WNoAutoErase)
-    , m_pixmap(CollectionDB::instance()->albumImage( artist, album, 0 ))
+CoverViewDialog::CoverViewDialog( const QString& artist, const QString& album, QWidget *parent )
+    : QDialog( parent, 0, false, WDestructiveClose | WType_TopLevel | WNoAutoErase )
+    , m_pixmap( CollectionDB::instance()->albumImage( artist, album, false, 0 ) )
 {
     KWin::setType( winId(), NET::Utility );
     kapp->setTopWidget( this );
-    setCaption( kapp->makeStdCaption( artist + i18n(" - ") + album ) );
+    setCaption( kapp->makeStdCaption( i18n("%1 - %2").arg( artist, album ) ) );
 
     m_layout = new QHBoxLayout( this );
-    m_layout->setAutoAdd(true);
+    m_layout->setAutoAdd( true );
     m_pixmapViewer = new PixmapViewer( this, m_pixmap );
 
     setFixedSize( m_pixmapViewer->maximalSize() );
@@ -357,6 +356,11 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
     if( item->depth() ) //album item
         return;
 
+    QString artist = item->text(0);
+
+    if( artist.endsWith( ", The" ) )
+        CollectionView::instance()->manipulateThe( artist, false );
+
     m_coverView->clear();
     m_coverItems.clear();
 
@@ -393,7 +397,7 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
     qb.setOptions( QueryBuilder::optNoCompilations );
 
     if ( item != m_artistView->firstChild() )
-        qb.addMatches( QueryBuilder::tabArtist, item->text( 0 ) );
+        qb.addMatches( QueryBuilder::tabArtist, artist );
 
     albums = qb.run();
 
@@ -426,7 +430,8 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
     //doing it in the second loop looks really bad, unfortunately
     //this is the slowest step in the bit that we can't process events
     uint x = 0;
-    for( QStringList::ConstIterator it = albums.begin(), end = albums.end(); it != end; ++it ) {
+    foreach( albums )
+    {
         const QString artist = *it;
         const QString album = *(++it);
         m_coverItems.append( new CoverViewItem( m_coverView, m_coverView->lastItem(), artist, album ) );
@@ -469,26 +474,26 @@ void CoverManager::showCoverMenu( QIconViewItem *item, const QPoint &p ) //SLOT
     QPtrList<CoverViewItem> selected = selectedItems();
     if( selected.count() > 1 ) {
         #ifdef AMAZON_SUPPORT
-        menu.insertItem( SmallIconSet( "www" ), i18n( "&Fetch Selected Covers" ), FETCH );
+        menu.insertItem( SmallIconSet( Amarok::icon( "download" ) ), i18n( "&Fetch Selected Covers" ), FETCH );
         #endif
-        menu.insertItem( SmallIconSet("folder_image"), i18n("Set &Custom Cover for Selected Albums"), CUSTOM );
-        menu.insertItem( SmallIconSet("editdelete"), i18n("&Unset Selected Covers"), DELETE );
+        menu.insertItem( SmallIconSet( Amarok::icon( "files" ) ), i18n("Set &Custom Cover for Selected Albums"), CUSTOM );
+        menu.insertItem( SmallIconSet( Amarok::icon( "remove" ) ), i18n("&Unset Selected Covers"), DELETE );
     }
     else {
-        menu.insertItem( SmallIconSet("viewmag"), i18n("&Show Fullsize"), SHOW );
+        menu.insertItem( SmallIconSet( Amarok::icon( "search" ) ), i18n("&Show Fullsize"), SHOW );
 
         #ifdef AMAZON_SUPPORT
-        menu.insertItem( SmallIconSet("www"), i18n("&Fetch From amazon.%1").arg( CoverManager::amazonTld() ), FETCH );
+        menu.insertItem( SmallIconSet( Amarok::icon( "download" ) ), i18n("&Fetch From amazon.%1").arg( CoverManager::amazonTld() ), FETCH );
         menu.insertSeparator();
-        menu.insertItem( SmallIconSet("folder_image"), i18n("Set &Custom Image"), CUSTOM );
+        menu.insertItem( SmallIconSet( Amarok::icon( "files" ) ), i18n("Set &Custom Image"), CUSTOM );
         #else
-        menu.insertItem( SmallIconSet("folder_image"), i18n("Set &Cover Image"), CUSTOM );
+        menu.insertItem( SmallIconSet( Amarok::icon( "files" ) ), i18n("Set &Cover Image"), CUSTOM );
         #endif
         menu.insertSeparator();
-        menu.insertItem( SmallIconSet("editdelete"), i18n("&Unset Cover Image"), DELETE );
+        menu.insertItem( SmallIconSet( Amarok::icon( "remove" ) ), i18n("&Unset Cover Image"), DELETE );
 
         menu.setItemEnabled( SHOW, item->hasCover() );
-        menu.setItemEnabled( DELETE, item->hasCover() );
+        menu.setItemEnabled( DELETE, item->canRemoveCover() );
     }
 
     switch( menu.exec(p) ) {
@@ -540,14 +545,16 @@ void CoverManager::slotSetFilter() //SLOT
 
     m_coverView->selectAll( false);
     QIconViewItem *item = m_coverView->firstItem();
-    while ( item ) {
+    while ( item )
+    {
         QIconViewItem *tmp = item->nextItem();
         m_coverView->takeItem( item );
         item = tmp;
     }
 
     m_coverView->setAutoArrange( false );
-    for( QIconViewItem *item = m_coverItems.first(); item; item = m_coverItems.next() ) {
+    for( QIconViewItem *item = m_coverItems.first(); item; item = m_coverItems.next() )
+    {
         CoverViewItem *coverItem = static_cast<CoverViewItem*>(item);
         if( coverItem->album().contains( m_filter, false ) || coverItem->artist().contains( m_filter, false ) )
             m_coverView->insertItem( item, m_coverView->lastItem() );
@@ -608,27 +615,8 @@ void CoverManager::changeView( int id  ) //SLOT
 
 void CoverManager::changeLocale( int id ) //SLOT
 {
-    switch ( id )
-    {
-        case International:
-            AmarokConfig::setAmazonLocale( "us" );
-            break;
-        case Canada:
-            AmarokConfig::setAmazonLocale( "ca" );
-            break;
-        case France:
-            AmarokConfig::setAmazonLocale( "fr" );
-            break;
-        case Germany:
-            AmarokConfig::setAmazonLocale( "de" );
-            break;
-        case Japan:
-            AmarokConfig::setAmazonLocale( "jp" );
-            break;
-        case UK:
-            AmarokConfig::setAmazonLocale( "uk" );
-            break;
-    }
+    QString locale = CoverFetcher::localeIDToString( id );
+    AmarokConfig::setAmazonLocale( locale );
     m_amazonLocaleMenu->setItemChecked( m_currentLocale, false );
     m_amazonLocaleMenu->setItemChecked( id, true );
     m_currentLocale = id;
@@ -798,9 +786,10 @@ void CoverManager::updateStatusBar()
             QStringList values = QStringList::split( " @@@ ", m_fetchCovers[0], true );    //get artist and album name
             if ( values.count() >= 2 )
             {
-                text = i18n( "Fetching cover for " );
-        if ( !values[0].isEmpty() ) text += values[0] + i18n(" - ");
-        text += values[1] + i18n("...");
+                if( values[0].isEmpty() )
+                    text = i18n( "Fetching cover for %1..." ).arg( values[1] );
+                else
+                    text = i18n( "Fetching cover for %1 - %2..." ).arg( values[0], values[1] );
             }
         }
         else if( m_fetchingCovers ) {
@@ -827,7 +816,7 @@ void CoverManager::updateStatusBar()
         //album info
         for( QIconViewItem *item = m_coverView->firstItem(); item; item = item->nextItem() ) {
             totalCounter++;
-            if( !((CoverViewItem*)item)->hasCover() )
+            if( !static_cast<CoverViewItem*>( item )->hasCover() )
                 missingCounter++;    //counter for albums without cover
         }
 
@@ -836,7 +825,12 @@ void CoverManager::updateStatusBar()
         else if( m_artistView->selectedItem() ) {
             text = i18n( "1 album", "%n albums", totalCounter );
             if( m_artistView->selectedItem() != m_artistView->firstChild() ) //showing albums by an artist
-                text += i18n( " by " ) + m_artistView->selectedItem()->text(0);
+            {
+                QString artist = m_artistView->selectedItem()->text(0);
+                if( artist.endsWith( ", The" ) )
+                    CollectionView::instance()->manipulateThe( artist, false );
+                text += i18n( " by " ) + artist;
+            }
         }
 
         if( missingCounter )
@@ -887,14 +881,14 @@ QDragObject *CoverView::dragObject()
     if( !item )
        return 0;
 
-    const QString sql = "SELECT tags.url FROM tags, album WHERE album.name LIKE '%1' AND tags.album = album.id ORDER BY tags.track;";
-    const QStringList values = CollectionDB::instance()->query( sql.arg( item->album() ) );
+    const QString sql = "SELECT tags.url FROM tags, album WHERE album.name %1 AND tags.album = album.id ORDER BY tags.track;";
+    const QStringList values = CollectionDB::instance()->query( sql.arg( CollectionDB::likeCondition( item->album() ) ) );
 
     KURL::List urls;
     for( QStringList::ConstIterator it = values.begin(), end = values.end(); it != end; ++it )
         urls += *it;
 
-    QString imagePath = CollectionDB::instance()->albumImage( item->artist(), item->album(), 1 );
+    QString imagePath = CollectionDB::instance()->albumImage( item->artist(), item->album(), false, 1 );
     KMultipleDrag *drag = new KMultipleDrag( this );
     drag->setPixmap( item->coverPixmap() );
     drag->addDragObject( new QIconDrag( this ) );
@@ -931,7 +925,7 @@ CoverViewItem::CoverViewItem( QIconView *parent, QIconViewItem *after, const QSt
     : KIconViewItem( parent, after, album )
     , m_artist( artist )
     , m_album( album )
-    , m_coverImagePath( CollectionDB::instance()->albumImage( m_artist, m_album, 0 ) )
+    , m_coverImagePath( CollectionDB::instance()->albumImage( m_artist, m_album, false, 0, &m_embedded ) )
     , m_coverPixmap( 0 )
 {
     setDragEnabled( true );
@@ -946,7 +940,7 @@ bool CoverViewItem::hasCover() const
 
 void CoverViewItem::loadCover()
 {
-    m_coverImagePath = CollectionDB::instance()->albumImage( m_artist, m_album );
+    m_coverImagePath = CollectionDB::instance()->albumImage( m_artist, m_album, false, 1, &m_embedded );
     m_coverPixmap = QPixmap( m_coverImagePath );  //create the scaled cover
 
     repaint();
@@ -992,15 +986,7 @@ void CoverViewItem::paintItem(QPainter* p, const QColorGroup& cg)
     int nameWidth = fm.width( str );
     if( nameWidth > textRect().width() )
     {
-        QString nameJustify = i18n("...");
-        int i = 0;
-        while ( fm.width( nameJustify + str[ i ] ) < textRect().width() )
-            nameJustify += str[ i++ ];
-        nameJustify.remove( 0, 3 );
-        if ( nameJustify.isEmpty() )
-            nameJustify = str.left( 1 );
-    nameJustify += i18n("...");
-        str = nameJustify;
+        str = KStringHandler::rPixelSqueeze( str, p->fontMetrics(), textRect().width() );
     }
     p->setPen( cg.text() );
     p->drawText( textRect(), Qt::AlignCenter, str );
@@ -1031,7 +1017,7 @@ void CoverViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem> & )
        QImage img;
        QImageDrag::decode( e, img );
        CollectionDB::instance()->setAlbumImage( artist(), album(), img );
-       m_coverImagePath = CollectionDB::instance()->albumImage( m_artist, m_album, 0 );
+       m_coverImagePath = CollectionDB::instance()->albumImage( m_artist, m_album, false, 0 );
        loadCover();
     }
 }
