@@ -35,7 +35,6 @@ AMAROK_EXPORT_PLUGIN( XineEngine )
 
 #include <qapplication.h>
 #include <qdir.h>
-#include <qtimer.h>
 
 extern "C"
 {
@@ -64,6 +63,7 @@ namespace Log
 //static inline QCString configPath() { return QFile::encodeName(KStandardDirs().localkdedir() + KStandardDirs::kde_default("data") + "amarok/xine-config"); }
 static inline QCString configPath() { return QFile::encodeName(locate( "data", "amarok/") + "xine-config" ); }
 static Fader *s_fader = 0;
+static OutFader *s_outfader = 0;
 
 
 XineEngine::XineEngine()
@@ -75,6 +75,7 @@ XineEngine::XineEngine()
         , m_post( 0 )
         , m_preamp( 1.0 )
         , m_stopFader( false )
+        , m_fadeOutRunning ( false )
         , m_equalizerEnabled( false )
 {
     addPluginProperty( "HasConfigure", "true" );
@@ -91,28 +92,17 @@ XineEngine::~XineEngine()
     // Wait until the fader thread is done
     if( s_fader ) {
         m_stopFader = true;
+        s_fader->resume(); // safety call if the engine is in the pause state
         s_fader->wait();
         delete s_fader;
     }
-
-    // NOTE The fadeout gets stuck when the EQ is active, so we skip it then
-    if( !m_equalizerEnabled && m_stream && state() == Engine::Playing )
-    {
-        const int volume = xine_get_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL );
-        const double D = 300000 * std::pow( (double)volume, -0.4951 );
-
-        debug() << "Sleeping: " << D << ", " << volume << endl;
-
-        for( int v = volume - 1; v >= 1; v-- ) {
-            xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, v );
-
-            const int sleep = int(D * (-log10( v + 1 ) + 2));
-
-            ::usleep( sleep );
-        }
-        xine_stop( m_stream );
+    if( s_outfader ) {
+        s_outfader->wait();
+        delete s_outfader;
     }
 
+    fadeOut( true ); // true == exiting
+
     if( m_xine )       xine_config_save( m_xine, configPath() );
 
     if( m_stream )     xine_close( m_stream );
@@ -226,8 +216,18 @@ XineEngine::load( const KURL &url, bool isStream )
 
     Engine::Base::load( url, isStream );
 
-    if( m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
+    if( m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY &&
+        ( m_xfadeNextTrack || //set by engine controller when switching tracks automatically
+         (uint) AmarokConfig::crossfadeType() == 0 ||  //crossfade always
+         (uint) AmarokConfig::crossfadeType() == 2 ) ) //crossfade when switching tracks manually
     {
+       m_xfadeNextTrack = false;
+       // Stop a probably running fader
+       if( s_fader ) {
+           m_stopFader = true;
+           s_fader->terminate(); // kills a running fader
+           delete s_fader;
+       }
        s_fader = new Fader( this, m_xfadeLength );
        setEqualizerParameters( m_intPreamp, m_equalizerGains );
     }
@@ -342,7 +342,7 @@ XineEngine::determineAndShowErrorMessage()
                 // xine can read the plugin but it didn't find any codec
                 // THUS xine=daft for telling us it could handle the format in canDecode!
                 body = i18n("There is no available decoder.");
-                QString const ext = amaroK::extension( m_url.url() ).lower();
+                QString const ext = Amarok::extension( m_url.url() ).lower();
                 if (ext == "mp3" && EngineController::installDistroCodec( "xine-engine" ))
                     return;
             }
@@ -351,7 +351,7 @@ XineEngine::determineAndShowErrorMessage()
         break;
     }
 
-    amaroK::StatusBar::instance()->longMessage(
+    Amarok::StatusBar::instance()->longMessage(
             "<b>" + i18n("Error Loading Media") + "</b><p>" + body + "<p>" + m_url.prettyURL(),
             KDE::StatusBar::Error );
 }
@@ -359,31 +359,44 @@ XineEngine::determineAndShowErrorMessage()
 void
 XineEngine::stop()
 {
+    if( s_fader && s_fader->running() )
+        s_fader->resume(); // safety call if the engine is in the pause state
+
     if ( !m_stream )
        return;
+    if( !m_fadeOutRunning || state() == Engine::Paused )
+    {
+        s_outfader = new OutFader( this, true, true );
+        s_outfader->start();
+        ::usleep( 100 ); //to be sure engine state won't be changed before it is checked in fadeOut()
+        m_url = KURL(); //to ensure we return Empty from state()
 
-    m_url = KURL(); //to ensure we return Empty from state()
+        std::fill( m_scope.begin(), m_scope.end(), 0 );
 
-    std::fill( m_scope.begin(), m_scope.end(), 0 );
-
-    xine_stop( m_stream );
-    xine_close( m_stream );
-    xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
-
-    emit stateChanged( Engine::Empty );
+        emit stateChanged( Engine::Empty );
+    }
 }
 
 void
 XineEngine::pause()
 {
-    if( xine_get_param( m_stream, XINE_PARAM_SPEED ) )
+    if ( !m_stream )
+        return;
+
+    if( xine_get_param( m_stream, XINE_PARAM_SPEED ) != XINE_SPEED_PAUSE )
     {
-        xine_set_param( m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE );
+        if( s_fader && s_fader->running() )
+            s_fader->pause();
+
+        xine_set_param( m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE ); 
         xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
         emit stateChanged( Engine::Paused );
 
     } else {
 
+        if( s_fader && s_fader->running() )
+            s_fader->resume();
+
         xine_set_param( m_stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL );
         emit stateChanged( Engine::Playing );
     }
@@ -392,12 +405,12 @@ XineEngine::pause()
 Engine::State
 XineEngine::state() const
 {
-    if ( !m_stream )
+    if ( !m_stream || m_fadeOutRunning )
        return Engine::Empty;
 
     switch( xine_get_status( m_stream ) )
     {
-    case XINE_STATUS_PLAY: return xine_get_param( m_stream, XINE_PARAM_SPEED ) ? Engine::Playing : Engine::Paused;
+    case XINE_STATUS_PLAY: return xine_get_param( m_stream, XINE_PARAM_SPEED )  != XINE_SPEED_PAUSE ? Engine::Playing : Engine::Paused;
     case XINE_STATUS_IDLE: return Engine::Empty;
     case XINE_STATUS_STOP:
     default:               return m_url.isEmpty() ? Engine::Empty : Engine::Idle;
@@ -481,6 +494,51 @@ XineEngine::setVolumeSW( uint vol )
         xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, static_cast<uint>( vol * m_preamp ) );
 }
 
+void
+XineEngine::fadeOut( bool exiting )
+{
+    if( m_fadeOutRunning ) //Let us not start another fadeout...
+        return;
+
+    m_fadeOutRunning = !m_fadeOutRunning;
+    const bool isPlaying = m_stream && ( xine_get_status( m_stream ) == XINE_STATUS_PLAY );
+    const float originalVol = Engine::Base::makeVolumeLogarithmic( m_volume ) * m_preamp;
+
+    // On shutdown, limit fadeout to 3 secs max, so that we don't risk getting killed
+    const int length = exiting ? QMIN( m_xfadeLength, 3000 ) : m_xfadeLength;
+
+    // NOTE The fadeout gets stuck when the EQ is active, so we skip it then
+    if( length > 0 && !m_equalizerEnabled && isPlaying )
+    {
+        // fader-class doesn't work in this spot as is, so some parts need to be copied here... (ugly)
+        uint stepsCount = length < 1000 ? length / 10 : 100;
+        uint stepSizeUs = (int)( 1000.0 * (float)length / (float)stepsCount );
+
+        ::usleep( stepSizeUs );
+        QTime t;
+        t.start();
+        float mix = 0.0;
+        while ( mix < 1.0 )
+        {
+            ::usleep( stepSizeUs );
+            float vol = Engine::Base::makeVolumeLogarithmic( m_volume ) * m_preamp;
+            float mix = (float)t.elapsed() / (float)length;
+            if ( mix > 1.0 )
+            {
+                break;
+            }
+            if ( m_stream )
+            {
+                float v = 4.0 * (1.0 - mix) / 3.0;
+                xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)( v < 1.0 ? vol * v : vol ) );
+            }
+        }
+    }
+    if( m_fadeOutRunning && m_stream )
+        xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, (uint) originalVol );
+    m_fadeOutRunning = !m_fadeOutRunning;
+}
+
 void
 XineEngine::setEqualizerEnabled( bool enable )
 {
@@ -513,6 +571,9 @@ XineEngine::setEqualizerEnabled( bool enable )
 void
 XineEngine::setEqualizerParameters( int preamp, const QValueList<int> &gains )
 {
+    if ( !m_stream )
+        return;
+
     m_equalizerGains = gains;
     m_intPreamp = preamp;
     QValueList<int>::ConstIterator it = gains.begin();
@@ -683,7 +744,7 @@ XineEngine::timerEvent( QTimerEvent* )
    }
 }
 
-amaroK::PluginConfig*
+Amarok::PluginConfig*
 XineEngine::configure() const
 {
     XineConfigDialog* xcf = new XineConfigDialog( m_xine );
@@ -755,7 +816,7 @@ void XineEngine::configChanged()
         init();
         setEqualizerEnabled( m_equalizerEnabled );
         if( m_equalizerEnabled )
-                           setEqualizerParameters( m_intPreamp, m_equalizerGains );
+            setEqualizerParameters( m_intPreamp, m_equalizerGains );
         emit resetConfig(m_xine);
     }
 }
@@ -962,6 +1023,7 @@ Fader::Fader( XineEngine *engine, uint fadeMs )
    , m_port( engine->m_audioPort )
    , m_post( engine->m_post )
    , m_fadeLength( fadeMs )
+   , m_paused( false )
 {
     if( engine->makeNewStream() )
     {
@@ -1000,19 +1062,23 @@ Fader::run()
     uint stepsCount = m_fadeLength < 1000 ? m_fadeLength / 10 : 100;
     uint stepSizeUs = (int)( 1000.0 * (float)m_fadeLength / (float)stepsCount );
 
-    QTime t;
-    t.start();
     float mix = 0.0;
+    float elapsedUs = 0.0;
     while ( mix < 1.0 )
     {
         // sleep a constant amount of time
         QThread::usleep( stepSizeUs );
 
+        if ( m_paused )
+        	continue;
+
+        elapsedUs += stepSizeUs;
+
         // get volume (amarok main * equalizer preamp)
         float vol = Engine::Base::makeVolumeLogarithmic( m_engine->m_volume ) * m_engine->m_preamp;
 
         // compute the mix factor as the percentage of time spent since fade begun
-        float mix = (float)t.elapsed() / (float)m_fadeLength;
+        float mix = (elapsedUs / 1000.0) / (float)m_fadeLength;
         if ( mix > 1.0 )
         {
             if ( m_increase )
@@ -1038,11 +1104,65 @@ Fader::run()
     //stop using cpu!
     xine_stop( m_decrease );
 
-    QThread::sleep( 5 );
-
     deleteLater();
 }
 
+void
+Fader::pause()
+{
+	m_paused = true;
+}
+
+void
+Fader::resume()
+{
+	m_paused = false;
+}
+
+
+//////////////////
+/// class OutFader
+//////////////////
+
+OutFader::OutFader( XineEngine *engine, bool stop, bool force )
+   : QObject( engine )
+   , QThread()
+   , m_engine( engine )
+   , m_stop( stop )
+   , m_force( force )
+{
+}
+
+OutFader::~OutFader()
+{
+     wait();
+
+     DEBUG_FUNC_INFO
+
+     s_outfader = 0;
+}
+
+void
+OutFader::run()
+{
+    m_engine->fadeOut();
+    if( m_engine->m_fadeOutRunning == false || m_force )
+    {
+        if( m_stop )
+        {
+            xine_stop( m_engine->m_stream );
+            xine_close( m_engine->m_stream );
+            xine_set_param( m_engine->m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
+        }
+        else
+        {
+            xine_set_param( m_engine->m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE );
+            xine_set_param( m_engine->m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
+        }
+    }
+    QThread::sleep( 3 );
+    deleteLater();
+}
 
 bool XineEngine::metaDataForUrl(const KURL &url, Engine::SimpleMetaBundle &b)
 {
@@ -1071,6 +1191,8 @@ bool XineEngine::metaDataForUrl(const KURL &url, Engine::SimpleMetaBundle &b)
                 b.tracknr =
                     QString::fromUtf8(
                         xine_get_meta_info(tmpstream, XINE_META_INFO_TRACK_NUMBER));
+                if( b.tracknr.isEmpty() )
+                    b.tracknr = url.filename();
             } else {
                 b.title = QString(i18n("Track %1")).arg(url.filename());
                 b.album = i18n("AudioCD");
