@@ -17,19 +17,17 @@
 #include "engineobserver.h"
 #include "metabundle.h"
 
-#include <qguardedptr.h>
 #include <qmap.h>
 #include <qobject.h>
 #include <qvaluelist.h>
 
 class QTimer;
 
-namespace amaroK { class StreamProvider; }
 namespace KIO { class Job; }
 
 
 /**
- * This class captures amaroK specific behaviour for some common features.
+ * This class captures Amarok specific behaviour for some common features.
  * Accessing the engine directly is perfectly legal but on your own risk.
  * TODO: Hide proxy stuff!
  */
@@ -51,27 +49,31 @@ public:
     static QString           engineProperty( const QString& key ) { return engine()->pluginProperty( key ); }
     static bool              hasEngineProperty( const QString& key ) { return engine()->hasPluginProperty( key ); }
 
+    uint                     trackPosition() const;
+
     EngineBase* loadEngine();
 
     uint trackLength() const { return m_bundle.length() * 1000; }
     const MetaBundle &bundle() const;
-    const KURL &playingURL() const { return bundle().url(); }
+    KURL playingURL() const { return bundle().url(); }
 
     void restoreSession();
     void endSession();
 
-    void currentTrackMetaDataChanged( const MetaBundle& bundle ) { m_bundle = bundle; newMetaDataNotify( bundle, false /* no track change */ ); }
-
     //xx000, xx100, xx200, so at most will be 200ms delay before time displays are updated
     static const int MAIN_TIMER = 300;
 
+    /*enum Filetype { MP3 };*/ //assuming MP3 for time being
+    LIBAMAROK_EXPORT static bool installDistroCodec(const QString& engine /*Filetype type*/);
+
 public slots:
     void previous();
     // forceNext make we go to next track even if Repeat Track is on
     //NOTE If the track ended normally, call next(false) !
     void next( const bool forceNext = true );
+    void trackFinished() { next(false); };
     void play();
-    void play( const MetaBundle& );
+    void play( const MetaBundle&, uint offset = 0 );
     void pause();
     void stop();
     void playPause(); //pauses if playing, plays if paused or stopped
@@ -87,6 +89,9 @@ public slots:
 
     void mute();
 
+    void slotStreamMetaData( const MetaBundle &bundle );
+    void currentTrackMetaDataChanged( const MetaBundle& bundle ) { m_bundle = bundle; newMetaDataNotify( bundle, false /* no track change */ ); }
+
 signals:
     void orderPrevious();
     void orderCurrent();
@@ -94,14 +99,11 @@ signals:
     void statusText( const QString& );
 
 private slots:
-    void playRemote( KIO::Job* );
-    void slotStreamMetaData( const MetaBundle& );
     void slotEngineMetaData( const Engine::SimpleMetaBundle& );
     void slotMainTimer();
     void slotTrackEnded();
     void slotStateChanged( Engine::State );
-    void streamError();
-	void slotSigError();
+
 protected:
     EngineController();
    ~EngineController();
@@ -124,8 +126,10 @@ private:
     bool            m_xFadeThisTrack;
     bool            m_isTiming;
     QTimer*         m_timer;
-
-    QGuardedPtr<amaroK::StreamProvider> m_stream;
+    uint            m_playFailureCount;
+    // try to correct start time for tracks from last.fm streams
+    bool            m_lastFm;
+    uint            m_positionOffset, m_lastPositionOffset;
 };
 
 
