@@ -18,17 +18,21 @@
  *   Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307  *
  *   USA                                                                   *
  ***************************************************************************/
-
 #include "config.h"
 #include "debug.h"
+#include "statusbar.h"
 #define DEBUG_PREFIX "KTRM"
 
 #include "ktrm.h"
 
 #include <kapplication.h>
+#include <kio/job.h>
+#include <kio/jobclasses.h>
 #include <kprotocolmanager.h>
 #include <kurl.h>
+#include <kresolver.h>
 
+#include <qdom.h>
 #include <qmutex.h>
 #include <qevent.h>
 #include <qobject.h>
@@ -37,7 +41,11 @@
 
 #if HAVE_TUNEPIMP
 
+#if HAVE_TUNEPIMP >= 5
+#include <tunepimp-0.5/tp_c.h>
+#else
 #include <tunepimp/tp_c.h>
+#endif
 
 class KTRMLookup;
 
@@ -69,7 +77,6 @@ public:
     int startLookup(KTRMLookup *lookup)
     {
         int id;
-
         if(!m_fileMap.contains(lookup->file())) {
 #if HAVE_TUNEPIMP >= 4
             id = tp_AddFile(m_pimp, QFile::encodeName(lookup->file()), 0);
@@ -90,7 +97,10 @@ public:
     {
         tp_ReleaseTrack(m_pimp, tp_GetTrack(m_pimp, lookup->fileId()));
         tp_Remove(m_pimp, lookup->fileId());
+        m_lookupMapMutex.lock();
         m_lookupMap.remove(lookup->fileId());
+        m_fileMap.remove( lookup->file() );
+        m_lookupMapMutex.unlock();
     }
 
     bool lookupMapContains(int fileId) const
@@ -126,8 +136,10 @@ protected:
     {
         m_pimp = tp_New("KTRM", "0.1");
         //tp_SetDebug(m_pimp, true);
+#if HAVE_TUNEPIMP < 5
         tp_SetTRMCollisionThreshold(m_pimp, 100);
         tp_SetAutoFileLookup(m_pimp,true);
+#endif
         tp_SetAutoSaveThreshold(m_pimp, -1);
         tp_SetMoveFiles(m_pimp, false);
         tp_SetRenameFiles(m_pimp, false);
@@ -138,10 +150,46 @@ protected:
 #endif
         tp_SetNotifyCallback(m_pimp, TRMNotifyCallback, 0);
 
+#if HAVE_TUNEPIMP < 5
+        KProtocolManager::reparseConfiguration();
+
         if(KProtocolManager::useProxy()) {
-            KURL proxy = KProtocolManager::proxyFor("http");
-            tp_SetProxy(m_pimp, proxy.host().latin1(), short(proxy.port()));
+            QString noProxiesFor = KProtocolManager::noProxyFor();
+            QStringList noProxies = QStringList::split(QRegExp("[',''\t'' ']"), noProxiesFor);
+            bool useProxy = true;
+
+            char server[255];
+            short port;
+            tp_GetServer(m_pimp, server, 255, &port);
+            QString tunepimpHost = QString(server);
+            QString tunepimpHostWithPort = (tunepimpHost + ":%1").arg(port);
+
+            for(QStringList::ConstIterator it = noProxies.constBegin(); it != noProxies.constEnd(); ++it) {
+                QString normalizedHost = KNetwork::KResolver::normalizeDomain(*it);
+                if(normalizedHost == tunepimpHost ||
+                    tunepimpHost.endsWith('.' + normalizedHost)) {
+                    useProxy = false;
+                    break;
+                }
+
+                if(normalizedHost == tunepimpHostWithPort ||
+                    tunepimpHostWithPort.endsWith('.' + normalizedHost)) {
+                    useProxy = false;
+                    break;
+                }
+            }
+
+            if(KProtocolManager::useReverseProxy())
+                useProxy = !useProxy;
+
+            if(useProxy) {
+                KURL proxy = KProtocolManager::proxyFor("http");
+                tp_SetProxy(m_pimp, proxy.host().latin1(), short(proxy.port()));
+            }
         }
+#else
+        tp_SetMusicDNSClientId(m_pimp, "0c6019606b1d8a54d0985e448f3603ca");
+#endif
     }
 
     ~KTRMRequestHandler()
@@ -168,6 +216,7 @@ public:
         Recognized,
         Unrecognized,
         Collision,
+        PuidGenerated,
         Error
     };
 
@@ -251,6 +300,9 @@ protected:
         case KTRMEvent::Collision:
             lookup->collision();
             break;
+        case KTRMEvent::PuidGenerated:
+            lookup->puidGenerated();
+            break;
         case KTRMEvent::Error:
             lookup->error();
             break;
@@ -259,7 +311,7 @@ protected:
 };
 
 /**
- * Callback fuction for TunePimp lookup events.
+ * Callback function for TunePimp lookup events.
  */
 #if HAVE_TUNEPIMP >= 4
 static void TRMNotifyCallback(tunepimp_t /*pimp*/, void */*data*/, TPCallbackEnum type, int fileId, TPFileStatus status)
@@ -275,6 +327,8 @@ static void TRMNotifyCallback(tunepimp_t pimp, void */*data*/, TPCallbackEnum ty
     TPFileStatus status = tr_GetStatus(track);
 #endif
 
+    debug() << "Status is: " << status << endl;
+
     switch(status) {
     case eRecognized:
         KTRMEventHandler::send(fileId, KTRMEvent::Recognized);
@@ -282,12 +336,20 @@ static void TRMNotifyCallback(tunepimp_t pimp, void */*data*/, TPCallbackEnum ty
     case eUnrecognized:
         KTRMEventHandler::send(fileId, KTRMEvent::Unrecognized);
         break;
+#if HAVE_TUNEPIMP >= 5
+    case ePUIDLookup:
+    case ePUIDCollision:
+    case eFileLookup:
+        KTRMEventHandler::send(fileId, KTRMEvent::PuidGenerated);
+        break;
+#else
     case eTRMCollision:
 #if HAVE_TUNEPIMP >= 4
     case eUserSelection:
 #endif
         KTRMEventHandler::send(fileId, KTRMEvent::Collision);
         break;
+#endif
     case eError:
         KTRMEventHandler::send(fileId, KTRMEvent::Error);
         break;
@@ -344,6 +406,8 @@ KTRMResult::KTRMResult(const KTRMResult &result)
 {
 #if HAVE_TUNEPIMP
     d = new KTRMResultPrivate(*result.d);
+#else
+    Q_UNUSED(result);
 #endif
 }
 
@@ -404,6 +468,7 @@ bool KTRMResult::operator<(const KTRMResult &r) const
 #if HAVE_TUNEPIMP
     return r.d->relevance < d->relevance;
 #else
+    Q_UNUSED(r);
     return false;
 #endif
 }
@@ -413,6 +478,7 @@ bool KTRMResult::operator>(const KTRMResult &r) const
 #if HAVE_TUNEPIMP
     return r.d->relevance > d->relevance;
 #else
+    Q_UNUSED(r);
     return true;
 #endif
 }
@@ -421,6 +487,8 @@ KTRMResult &KTRMResult::operator= (const KTRMResult &r)
 {
 #if HAVE_TUNEPIMP
     d = new KTRMResultPrivate(*r.d);
+#else
+    Q_UNUSED(r);
 #endif
     return *this;
 }
@@ -429,6 +497,8 @@ bool KTRMResult::operator== (const KTRMResult &r) const
 {
 #if HAVE_TUNEPIMP
     return *d == *(r.d);
+#else
+    Q_UNUSED(r);
 #endif
     return false;
 }
@@ -472,6 +542,9 @@ KTRMLookup::KTRMLookup(const QString &file, bool autoDelete)
     d->file = file;
     d->autoDelete = autoDelete;
     d->fileId = KTRMRequestHandler::instance()->startLookup(this);
+#else
+    Q_UNUSED(file);
+    Q_UNUSED(autoDelete);
 #endif
 }
 
@@ -539,7 +612,11 @@ void KTRMLookup::unrecognized()
     trm[0] = 0;
     track_t track = tp_GetTrack(KTRMRequestHandler::instance()->tunePimp(), d->fileId);
     tr_Lock(track);
+#if HAVE_TUNEPIMP >= 5
+    tr_GetPUID(track, trm, 255);
+#else
     tr_GetTRM(track, trm, 255);
+#endif
     if ( !trm[0] ) {
         tr_SetStatus(track, ePending);
         tp_Wake(KTRMRequestHandler::instance()->tunePimp(), track);
@@ -559,7 +636,7 @@ void KTRMLookup::unrecognized()
 
 void KTRMLookup::collision()
 {
-#if HAVE_TUNEPIMP
+#if HAVE_TUNEPIMP && HAVE_TUNEPIMP < 5
     debug() << k_funcinfo << d->file << endl;
 
     track_t track = tp_GetTrack(KTRMRequestHandler::instance()->tunePimp(), d->fileId);
@@ -598,7 +675,7 @@ void KTRMLookup::collision()
         case eTrackList:
         {
             debug() << "eTrackList" << endl;
-            albumtrackresult_t **tracks = (albumtrackresult_t **) results;
+            albumtrackresult_t **tracks = reinterpret_cast<albumtrackresult_t **>( results );
             d->results.clear();
 
             for(int i = 0; i < resultCount; i++) {
@@ -633,12 +710,101 @@ void KTRMLookup::collision()
     }
 
     tr_Unlock(track);
+    tp_ReleaseTrack(KTRMRequestHandler::instance()->tunePimp(), track);
     qHeapSort(d->results);
 
     finished();
 #endif
 }
 
+void KTRMLookup::puidGenerated()
+{
+#if HAVE_TUNEPIMP >= 5
+    DEBUG_BLOCK
+    debug() << k_funcinfo << d->file << endl;
+    char puid[255] = {0};
+    track_t track = tp_GetTrack(KTRMRequestHandler::instance()->tunePimp(), d->fileId);
+    tr_Lock(track);
+
+    tr_GetPUID(track, puid, 255);
+    debug() << puid << endl;
+    tr_Unlock(track);
+    tp_ReleaseTrack(KTRMRequestHandler::instance()->tunePimp(), track);
+    d->results.clear();
+
+    KIO::Job *job = KIO::storedGet( QString( "http://musicbrainz.org/ws/1/track/?type=xml&puid=%1" ).arg( puid ) , false, false );
+
+    Amarok::StatusBar::instance()->newProgressOperation( job )
+            .setDescription( i18n( "MusicBrainz Lookup" ) );
+
+    connect( job, SIGNAL( result( KIO::Job* ) ), SLOT( lookupResult( KIO::Job* ) ) );
+#endif
+}
+
+void KTRMLookup::lookupResult( KIO::Job* job )
+{
+#if HAVE_TUNEPIMP >= 5
+    DEBUG_BLOCK
+    if ( !job->error() == 0 ) {
+        warning() << "[MusicBrainzLookup] KIO error! errno: " << job->error() << endl;
+        Amarok::StatusBar::instance()->longMessage( "Couldn't connect to MusicBrainz server." );
+        finished();
+        return;
+    }
+    KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>( job );
+    QString xml = QString::fromUtf8( storedJob->data().data(), storedJob->data().size() );
+
+    QDomDocument doc;
+    QDomElement e;
+
+    if( !doc.setContent( xml ) ) {
+        warning() << "[MusicBrainzLookup] Invalid XML" << endl;
+        Amarok::StatusBar::instance()->longMessage( "MusicBrainz returned invalid content." );
+        finished();
+        return;
+    }
+
+    e = doc.namedItem( "metadata" ).toElement().namedItem( "track-list" ).toElement();
+
+    QStringList strList = QStringList::split ( '/', d->file );
+
+    QDomNode n = e.namedItem("track");
+    for( ; !n.isNull();  n = n.nextSibling() ) {
+        QDomElement track = n.toElement();
+        KTRMResult result;
+
+        result.d->title = track.namedItem( "title" ).toElement().text();
+        result.d->artist = track.namedItem( "artist" ).toElement().namedItem( "name" ).toElement().text();
+        QDomNode releaseNode = track.namedItem("release-list").toElement().namedItem("release");
+        for( ; !releaseNode.isNull();  releaseNode = releaseNode.nextSibling() ) {
+            KTRMResult tmpResult( result );
+            QDomElement release = releaseNode.toElement();
+
+            tmpResult.d->album = release.namedItem( "title" ).toElement().text();
+            QDomNode tracklistN = release.namedItem( "track-list" );
+            if ( !tracklistN.isNull() ) {
+                QDomElement tracklist = tracklistN.toElement();
+                if ( !tracklist.attribute( "offset" ).isEmpty() )
+                    tmpResult.d->track = tracklist.attribute( "offset" ).toInt() + 1;
+            }
+            //tmpResult.d->year = ???;
+            tmpResult.d->relevance =
+                4 * stringSimilarity(strList,tmpResult.d->title) +
+                2 * stringSimilarity(strList,tmpResult.d->artist) +
+                1 * stringSimilarity(strList,tmpResult.d->album);
+            if( !d->results.contains( tmpResult ) )
+                d->results.append( tmpResult );
+        }
+     }
+
+     qHeapSort(d->results);
+
+     finished();
+#else
+    Q_UNUSED( job );
+#endif
+}
+
 void KTRMLookup::error()
 {
 #if HAVE_TUNEPIMP
@@ -673,7 +839,7 @@ void KTRMLookup::finished()
     emit sigResult( results(), d->errorString );
 
     if(d->autoDelete)
-        delete this;
+        deleteLater();
 #endif
 }
 
