@@ -1,5 +1,5 @@
 /***************************************************************************
-                        loader.cpp  -  loader application for amaroK
+                        loader.cpp  -  loader application for Amarok
                            -------------------
   begin                : 2004/02/19
   copyright            : (C) 2004 by Mark Kretschmann
@@ -22,6 +22,9 @@
 #include <qmessagebox.h>
 #include <qprocess.h>
 #include <qstring.h>
+#include <kinstance.h>
+#include <kglobal.h>
+#include <kstandarddirs.h>
 #include "splash.h"
 
 extern "C"
@@ -45,7 +48,7 @@ main( int argc, char *argv[] )
 
     if( !args.isEmpty() )
     {
-        // These arguments cannot be passed to amaroK, or amaroK will exit
+        // These arguments cannot be passed to Amarok, or Amarok will exit
         // after processing them. TODO handle -v short hand for version
         QStringList longs; longs
                 << "-help" << "-help-qt" << "-help-kde" << "-help-all" << "-author" << "-version" << "-license";
@@ -82,10 +85,6 @@ main( int argc, char *argv[] )
         // there is no amarokapp running, we'll start a new
         // instance and the above checks were not necessary
     }
-    else if( isRunning )
-        // we are running, but no arguments were passed
-        // show and raise amarok
-        args = "--toggle-playlist-window";
 
     if ( isRunning ) {
         QStringList dcop_args;
@@ -151,15 +150,15 @@ Loader::Loader( QStringList args )
     m_proc = new QProcess( args, this );
     m_proc->setCommunication( QProcess::Stdout );
 
-    std::cout << "amaroK: [Loader] Starting amarokapp..\n";
-    std::cout << "amaroK: [Loader] Don't run gdb, valgrind, etc. against this binary! Use amarokapp.\n";
+    std::cout << "Amarok: [Loader] Starting amarokapp..\n";
+    std::cout << "Amarok: [Loader] Don't run gdb, valgrind, etc. against this binary! Use amarokapp.\n";
 
     if( !m_proc->start() )
     {
         delete m_splash; // hide the splash
 
-        QMessageBox::critical( 0, "amaroK",
-                "amaroK could not be started!\n" //FIXME this needs to be translated
+        QMessageBox::critical( 0, "Amarok",
+                "Amarok could not be started!\n" //FIXME this needs to be translated
                     "This may be because the amarokapp binary is not in your PATH.\n"
                     "Try locating and running amarokapp from a terminal.",
                 QMessageBox::Ok, 0 );
@@ -185,7 +184,7 @@ Loader::timerEvent( QTimerEvent* )
     {
          if( ++m_counter == (30000 / INTERVAL) )
              // 30 seconds have passed
-             std::cerr << "amaroK: [Loader] amaroK is taking a long time to load! Perhaps something has gone wrong?\n";
+             std::cerr << "Amarok: [Loader] Amarok is taking a long time to load! Perhaps something has gone wrong?\n";
 
         while( m_proc->canReadLineStdout() )
             if( m_proc->readLineStdout() == "STARTUP" )
@@ -193,7 +192,7 @@ Loader::timerEvent( QTimerEvent* )
     }
     else if( !m_proc->normalExit() ) {
         // no reason to show messagebox, as amarokapp should start drkonqi
-        std::cerr << "amaroK: [Loader] amarokapp probably crashed!\n";
+        std::cerr << "Amarok: [Loader] amarokapp probably crashed!\n";
 
         QApplication::exit( 3 );
     }
@@ -208,21 +207,29 @@ bool
 isSplashEnabled()
 {
     //determine whether splash-screen is enabled in amarokrc
+    KInstance instance("amarok"); // KGlobal::dirs() crashes without
+    (void)KGlobal::config(); // the kubuntu special directory is not present without this
+    QStringList dirs = KGlobal::dirs()->findAllResources( "config", "amarokrc" );
 
-    QString path( ::getenv( "KDEHOME" ) );
-    if ( path.isEmpty() )
-        path = ::getenv( "HOME" ) + QString("/.kde");
-    path += "/share/config/amarokrc";
-
-    QFile file( path );
-    if ( file.open( IO_ReadOnly ) ) {
-        QString line;
-        while( file.readLine( line, 2000 ) != -1 )
-            if ( line.contains( "Show Splashscreen" ) && line.contains( "false" ) )
-                return false;
+    for( QStringList::iterator path = dirs.begin();
+            path != dirs.end();
+            ++path )
+    {
+        QFile file( *path );
+        if ( file.open( IO_ReadOnly ) )
+        {
+            QString line;
+            while( file.readLine( line, 2000 ) != -1 )
+                if ( line.contains( "Show Splashscreen" ) )
+                {
+                    if( line.contains( "false" ) )
+                        return false;
+                    else
+                        return true;
+                }
+        }
     }
 
-    //if we fail to open it, just show the splash
-
+    //if we fail to open anything, just show the splash
     return true;
 }
