@@ -45,12 +45,8 @@ email                : markey@web.de
 #include <kurldrag.h>
 #include <kwin.h>            //eventFilter()
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-
-
 //simple function for fetching amarok images
-namespace amaroK
+namespace Amarok
 {
     //TODO remove these, they suck, do a generic getImage
 
@@ -69,7 +65,7 @@ namespace amaroK
     }
 }
 
-using amaroK::getPNG;
+using Amarok::getPNG;
 
 
 //fairly pointless template which was designed to make the ctor clearer,
@@ -107,7 +103,7 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
 
     setModifiedPalette();
     setFixedSize( 311, 140 );
-    setCaption( "amaroK" );
+    setCaption( "Amarok" );
     setAcceptDrops( true );
 
     //another quit shortcut because the other window has all the accels
@@ -124,7 +120,7 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
         //NOTE we use a layout for the buttons so resizing will be possible
         m_pFrameButtons = createWidget<QHBox>( QRect(0, 118, 311, 22), this );
 
-        KActionCollection *ac =amaroK::actionCollection();
+        KActionCollection *ac =Amarok::actionCollection();
 
         //FIXME change the names of the icons to reflect kde names so we can fall back to them if necessary
                          new NavButton( m_pFrameButtons, "prev", ac->action( "prev" ) );
@@ -144,10 +140,13 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
     } //</NavButtons>
 
     { //<Sliders>
-        m_pSlider    = new amaroK::PrettySlider( Qt::Horizontal, this );
-        m_pVolSlider = new amaroK::PrettySlider( Qt::Vertical, this, amaroK::VOLUME_MAX );
+        m_pSlider    = new Amarok::PrettySlider( Qt::Horizontal,
+                                                 Amarok::PrettySlider::Pretty, this );
+        m_pVolSlider = new Amarok::PrettySlider( Qt::Vertical,
+                                                 Amarok::PrettySlider::Pretty, this,
+                                                 Amarok::VOLUME_MAX );
 
-        m_pSlider->setGeometry( 4,103, 303,12 );
+        m_pSlider->setGeometry( 4,103, 303, 12 );
         m_pVolSlider->setGeometry( 294,18, 12,79 );
         m_pVolSlider->setValue( AmarokConfig::masterVolume() );
 
@@ -242,7 +241,7 @@ void PlayerWidget::setScroll( const QStringList &list )
     }
 
     //FIXME empty QString would crash due to NULL Pixmaps
-    if( text.isEmpty() ) text = i18n( "Please report this message to amarok-devel@lists.sf.net, thanks!" );
+    if( text.isEmpty() ) text = i18n( "Please report this message to amarok@kde.org, thanks!" );
 
     QFont font( m_pScrollFrame->font() );
     QFontMetrics fm( font );
@@ -264,7 +263,7 @@ void PlayerWidget::setScroll( const QStringList &list )
     {
         p.drawText( x, baseline, *it );
         x += fm.width( *it );
-        p.fillRect( x + 8, separatorYPos, 4, 4, amaroK::ColorScheme::Foreground );
+        p.fillRect( x + 8, separatorYPos, 4, 4, Amarok::ColorScheme::Foreground );
         x += separatorWidth;
     }
 
@@ -321,11 +320,12 @@ void PlayerWidget::engineStateChanged( Engine::State state, Engine::State /*oldS
             m_pSlider->setValue( 0 );
             m_pSlider->setMinValue( 0 );
             m_pSlider->setMaxValue( 0 );
+            m_pSlider->newBundle( MetaBundle() ); // Set an empty bundle for no moodbar
             m_pTimeLabel->hide();
             m_pTimeSign->hide();
             m_rateString = QString::null;
             m_pSlider->setEnabled( false );
-            setScroll( i18n( "Welcome to amaroK" ) );
+            setScroll( i18n( "Welcome to Amarok" ) );
             update();
             break;
 
@@ -362,13 +362,16 @@ void PlayerWidget::engineNewMetaData( const MetaBundle &bundle, bool )
     m_pSlider->setMinValue( 0 ); // Important. minValue could have been changed by bogus maxValues
     m_pSlider->setMaxValue( bundle.length() * 1000 );
     m_pSlider->setEnabled( bundle.length() > 0 );
+    m_pSlider->newBundle( bundle );
 
     m_rateString     = bundle.prettyBitrate();
-    const QString Hz = bundle.prettySampleRate( true );
+    QString Hz = bundle.prettySampleRate( true );
     if( !Hz.isEmpty() )
     {
-        if( !m_rateString.isEmpty() ) m_rateString += " - ";
-        m_rateString += Hz;
+        if( m_rateString.isEmpty() )
+            m_rateString = Hz;
+        else
+            m_rateString = i18n("%1 kBit - %2").arg( m_rateString, Hz );
     }
 
     QStringList list( bundle.prettyTitle() );
@@ -388,6 +391,12 @@ void PlayerWidget::engineTrackPositionChanged( long position, bool /*userSeek*/
 }
 
 
+void PlayerWidget::engineTrackLengthChanged( long length )
+{
+    m_pSlider->setMaxValue( length * 1000 );
+}
+
+
 void PlayerWidget::timeDisplay( int ms )
 {
     int seconds = ms / 1000;
@@ -424,7 +433,7 @@ void PlayerWidget::determineAmarokColors() //static
         : KGlobalSettings::highlightColor()
       ).getHsv( &hue, &s, &v );
 
-    using namespace amaroK::ColorScheme;
+    using namespace Amarok::ColorScheme;
 
     Text       = Qt::white;
     Background = comodulate( hue, 0x002090 );
@@ -441,15 +450,15 @@ void PlayerWidget::determineAmarokColors() //static
            hue = h;
     }
 
-    Base = comodulate( hue, amaroK::blue );
+    Base = comodulate( hue, Amarok::blue );
 }
 
 void PlayerWidget::setModifiedPalette()
 {
     QPalette p = QApplication::palette();
     QColorGroup cg = p.active();
-    cg.setColor( QColorGroup::Background, amaroK::ColorScheme::Base );
-    cg.setColor( QColorGroup::Foreground, amaroK::ColorScheme::Text );
+    cg.setColor( QColorGroup::Background, Amarok::ColorScheme::Base );
+    cg.setColor( QColorGroup::Foreground, Amarok::ColorScheme::Text );
     setPalette( QPalette(cg, p.disabled(), cg) );
 }
 
@@ -467,12 +476,15 @@ void PlayerWidget::applySettings()
     //update the scroller
     switch( EngineController::engine()->state() ) {
     case Engine::Empty:
-        m_scrollTextPixmap.fill( amaroK::ColorScheme::Base );
+        m_scrollTextPixmap.fill( Amarok::ColorScheme::Base );
         update();
         break;
     default:
         engineNewMetaData( EngineController::instance()->bundle(), false );
     }
+
+    if(m_pAnalyzer)
+        setMinimalView(m_minimalView);
 }
 
 void PlayerWidget::setMinimalView( bool enable )
@@ -488,18 +500,20 @@ void PlayerWidget::setMinimalView( bool enable )
     if( enable )
     {
         uint space = 2;
-        m_pScrollFrame->setGeometry ( 6,space,  m_pScrollFrame->width(),  m_pScrollFrame->height() );
-        m_pSlider->setGeometry      ( 4,space + m_pScrollFrame->height(), 303,12 );
+        m_pScrollFrame->setGeometry ( 6,space,  m_pScrollFrame->width(), m_pScrollFrame->height() );
+        m_pSlider->setGeometry      ( 4,space + m_pScrollFrame->height(), 303, 12 );
         m_pFrameButtons->setGeometry( 0,space + m_pScrollFrame->height() + m_pSlider->height(), 311,22 );
         uint height = m_pFrameButtons->height() + m_pScrollFrame->height() + m_pSlider->height() + space;
         setFixedSize( 311, height );
+        AmarokConfig::setPlayerWindowMinimalView( true );
     }
     else
     {
         m_pScrollFrame->setGeometry( 6,18, m_pScrollFrame->width(),m_pScrollFrame->height() );
         m_pSlider->setGeometry( 4,103, 303,12 );
-        m_pFrameButtons->setGeometry(0, 118, 311,22);
+        m_pFrameButtons->setGeometry(0,118, 311,22);
         setFixedSize( 311, 140 );
+        AmarokConfig::setPlayerWindowMinimalView( false );
     }
 
     m_minimalView = enable;
@@ -520,7 +534,7 @@ bool PlayerWidget::event( QEvent *e )
     case QEvent::Drop:
     case QEvent::Close:
 
-        amaroK::genericEventHandler( this, e );
+        Amarok::genericEventHandler( this, e );
         return true; //we handled it
 
     case QEvent::ApplicationPaletteChange:
@@ -622,7 +636,7 @@ bool PlayerWidget::event( QEvent *e )
             {
                 //if we have been iconified, iconify the Playlist Window too
                 //if we have been shaded, hide the PlaylistWindow
-                //if the user is on another desktop to amaroK, do nothing
+                //if the user is on another desktop to Amarok, do nothing
 
                 const KWin::WindowInfo info = KWin::windowInfo( winId() );
 
@@ -634,7 +648,7 @@ bool PlayerWidget::event( QEvent *e )
 
             }
             else
-                //we caused amaroK to hide, so we should hide the Playlist Window
+                //we caused Amarok to hide, so we should hide the Playlist Window
                 //NOTE we "override" closeEvents and thus they count as non-spontaneous
                 //hideEvents; which frankly is a huge relief!
                 parentWidget()->hide();
@@ -743,7 +757,7 @@ void PlayerWidget::mousePressEvent( QMouseEvent *e )
 {
     if ( e->button() == QMouseEvent::RightButton )
     {
-        amaroK::Menu::instance()->exec( e->globalPos() );
+        Amarok::Menu::instance()->exec( e->globalPos() );
     }
     else if ( m_pAnalyzer->geometry().contains( e->pos() ) )
     {
@@ -758,7 +772,7 @@ void PlayerWidget::mousePressEvent( QMouseEvent *e )
         if ( rect.contains( e->pos() ) )
         {
             AmarokConfig::setTimeDisplayRemaining( !AmarokConfig::timeDisplayRemaining() );
-            timeDisplay( EngineController::engine()->position() / 1000 );
+            timeDisplay( EngineController::engine()->position() );
         }
         else m_startDragPos = e->pos();
     }
@@ -791,7 +805,6 @@ void PlayerWidget::createAnalyzer( int increment )
 
 }
 
-
 void PlayerWidget::startDrag()
 {
     QDragObject *d = new QTextDrag( EngineController::instance()->bundle().prettyTitle(), this );
@@ -800,14 +813,6 @@ void PlayerWidget::startDrag()
 }
 
 
-void PlayerWidget::setEffectsWindowShown( bool on )
-{
-    m_pButtonEq->blockSignals( true );
-    m_pButtonEq->setOn( on );
-    m_pButtonEq->blockSignals( false );
-}
-
-
 void PlayerWidget::slotShowEqualizer( bool show ) //SLOT
 {
     if( show )
