@@ -70,7 +70,7 @@ class Medium;
  */
 
 
-class Playlist : private KListView, public EngineObserver, public amaroK::ToolTipClient
+class Playlist : private KListView, public EngineObserver, public Amarok::ToolTipClient
 {
         Q_OBJECT
 
@@ -147,12 +147,13 @@ class Playlist : private KListView, public EngineObserver, public amaroK::ToolTi
 
         int  numVisibleColumns() const;
         QValueList<int> visibleColumns() const;
+        MetaBundle::ColumnMask getVisibleColumnMask() const;
         int  mapToLogicalColumn( int physical ) const; // Converts physical PlaylistItem column position to logical
         QString columnText( int c ) const { return KListView::columnText( c ); };
         void setColumns( QValueList<int> order, QValueList<int> visible );
 
         /** Call this to prevent items being removed from the playlist, it is mostly for internal use only
-         *  Dont forget to unlock() !! */
+         *  Don't forget to unlock() !! */
         void lock();
         void unlock();
 
@@ -162,9 +163,8 @@ class Playlist : private KListView, public EngineObserver, public amaroK::ToolTi
 
         //ATF-related functions
         bool checkFileStatus( PlaylistItem * item );
-        void addToUniqueMap( const QString uniqueid, PlaylistItem * item ) { m_uniqueMap[uniqueid] = item; }
-        void removeFromUniqueMap( const QString uniqueid )
-            { if( m_uniqueMap.contains( uniqueid ) ) m_uniqueMap.remove( uniqueid ); }
+        void addToUniqueMap( const QString uniqueid, PlaylistItem* item );
+        void removeFromUniqueMap( const QString uniqueid, PlaylistItem* item );
 
         enum RequestType { Prev = -1, Current = 0, Next = 1 };
         enum StopAfterMode { DoNotStop, StopAfterCurrent, StopAfterQueue, StopAfterOther };
@@ -176,8 +176,8 @@ class Playlist : private KListView, public EngineObserver, public amaroK::ToolTi
         friend class QueueLabel;
         friend class PlaylistWindow;
         friend class ColumnList;
-        friend void amaroK::DcopPlaylistHandler::removeCurrentTrack(); //calls removeItem() and currentTrack()
-        friend void amaroK::DcopPlaylistHandler::removeByIndex( int ); //calls removeItem()
+        friend void Amarok::DcopPlaylistHandler::removeCurrentTrack(); //calls removeItem() and currentTrack()
+        friend void Amarok::DcopPlaylistHandler::removeByIndex( int ); //calls removeItem()
         friend class TagWriter; //calls removeItem()
         friend void PlaylistWindow::init(); //setting up connections etc.
         friend TrackToolTip::TrackToolTip();
@@ -234,6 +234,9 @@ class Playlist : private KListView, public EngineObserver, public amaroK::ToolTi
         void adjustColumn( int n );
         void updateEntriesUrl( const QString &oldUrl, const QString &newUrl, const QString &uniqueid );
         void updateEntriesUniqueId( const QString &url, const QString &oldid, const QString &newid );
+        void updateEntriesStatusDeleted( const QString &absPath, const QString &uniqueid );
+        void updateEntriesStatusAdded( const QString &absPath, const QString &uniqueid );
+        void updateEntriesStatusAdded( const QMap<QString,QString> &map );
 
     protected:
         virtual void fontChange( const QFont &old );
@@ -254,9 +257,10 @@ class Playlist : private KListView, public EngineObserver, public amaroK::ToolTi
 
         void generateInfo(); //generates info for Random Albums
 
-        void queue( QListViewItem*, bool multi = false );
-           /* the only difference multi makes is whether it emits queueChanged(). (if multi, then no)
-              if you're queue()ing many items, consider passing true and emitting queueChanged() yourself. */
+        /* the only difference multi makes is whether it emits queueChanged(). (if multi, then no)
+           if you're queue()ing many items, consider passing true and emitting queueChanged() yourself. */
+        /* if invertQueue then queueing an already queued song dequeues it */
+        void queue( QListViewItem*, bool multi = false, bool invertQueue = true );
 
         void saveUndoState();
         void setDelayedFilter();                                           //after the delay is over
@@ -401,7 +405,7 @@ class Playlist : private KListView, public EngineObserver, public amaroK::ToolTi
 
         std::vector<double> m_columnFraction;
 
-        QMap<QString,PlaylistItem*> m_uniqueMap;
+        QMap<QString,QPtrList<PlaylistItem>*> m_uniqueMap;
         int m_oldRandom;
         int m_oldRepeat;
 };
