@@ -6,17 +6,20 @@
 #ifndef UrlLoader_H
 #define UrlLoader_H
 
+#include "amarok.h"
 #include "debug.h"        //stack allocated
 #include <qptrlist.h>
+#include <qxml.h>         //baseclass
 #include <kurl.h>         //KURL::List
 #include "metabundle.h"   //stack allocated
 #include "threadweaver.h" //baseclass
+#include "xmlloader.h"    //baseclass
 
-class QDomNode;
 class QListViewItem;
 class QTextStream;
 class PlaylistItem;
 class PLItemList;
+class XMLData;
 
 namespace KIO { class Job; }
 
@@ -36,49 +39,58 @@ class PlaylistFile
 public:
     PlaylistFile( const QString &path );
 
-    enum Format { M3U, PLS, XML, RAM, ASX, Unknown, NotPlaylist = Unknown };
+    enum Format { M3U, PLS, XML, RAM, SMIL, ASX, XSPF, Unknown, NotPlaylist = Unknown };
 
     /// the bundles from this playlist, they only contain
     /// the information that can be extracted from the playlists
     BundleList &bundles() { return m_bundles; }
 
+    /// the name of the playlist. often stored in the document (eg xspf) or derived from the filename
+    QString &title() { return m_title; }
+
     ///@return true if couldn't load the playlist's contents
     bool isError() const { return !m_error.isEmpty(); }
 
     /// if start returns false this has a translated error description
     QString error() const { return m_error; }
 
+
     static inline bool isPlaylistFile( const KURL &url ) { return isPlaylistFile( url.fileName() ); }
     static inline bool isPlaylistFile( const QString &fileName ) { return format( fileName ) != Unknown; }
     static inline Format format( const QString &fileName );
+    static QTime stringToTime(const QString&);
 
 protected:
     /// make these virtual if you need to
     bool loadM3u( QTextStream& );
     bool loadPls( QTextStream& );
     unsigned int loadPls_extractIndex( const QString &str ) const;
-    bool loadRealAudioRam( QTextStream& ); 
+    bool loadRealAudioRam( QTextStream& );
+    bool loadASX( QTextStream& );
+    bool loadSMIL( QTextStream& );
+    bool loadXSPF( QTextStream& );
     QString m_path;
     QString m_error;
     BundleList m_bundles;
+    QString m_title;
 };
 
 inline PlaylistFile::Format
 PlaylistFile::format( const QString &fileName )
 {
-    const QString ext = fileName.right( 4 ).lower();
+    const QString ext = Amarok::extension( fileName );
 
-    if( ext == ".m3u" ) return M3U;
-    if( ext == ".pls" ) return PLS;
-    if( ext == ".ram" ) return RAM;
-    if( ext == ".asx" ) return ASX;
-    if( ext == ".xml" ) return XML;
+    if( ext == "m3u" ) return M3U;
+    if( ext == "pls" ) return PLS;
+    if( ext == "ram" ) return RAM;
+    if( ext == "smil") return SMIL;
+    if( ext == "asx" || ext == "wax" ) return ASX;
+    if( ext == "xml" ) return XML;
+    if( ext == "xspf" ) return XSPF;
 
     return Unknown;
 }
 
-
-
 /**
  * @author Max Howell
  * @author Mark Kretschmann
@@ -109,6 +121,10 @@ protected:
 
     void loadXml( const KURL& );
 
+private slots:
+    void slotNewBundle( const MetaBundle &bundle, const XmlAttributeList &attributes );
+    void slotPlaylistInfo( const QString &product, const QString &version, const QString &dynamicMode );
+
 private:
     KURL::List recurse( const KURL& );
 
@@ -119,6 +135,10 @@ private:
     bool          m_playFirstUrl;
     Debug::Block  m_block;
     QPtrList<PlaylistItem> m_oldQueue;
+    QXmlInputSource  *m_xmlSource;
+    QValueList<XMLData> m_xml;
+    KURL m_currentURL;
+    QString m_dynamicMode;
 
 protected:
     UrlLoader( const UrlLoader& ); //undefined
@@ -168,5 +188,17 @@ private slots:
     void abort() { delete this; }
 };
 
+// PRIVATE -- should be in the .cpp, but fucking moc.
+
+class MyXmlLoader: public MetaBundle::XmlLoader
+{
+    Q_OBJECT
+public:
+    MyXmlLoader() { }
+    virtual bool startElement( const QString&, const QString&, const QString &, const QXmlAttributes& );
+signals:
+    void playlistInfo( const QString &product, const QString &version, const QString &dynamicMode );
+};
+
 
 #endif
