@@ -11,8 +11,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "amarok.h"
 #include "amarokconfig.h"     //check if dynamic mode
-#include "debug.h"
 #include "playlist.h"
 #include "queuemanager.h"
 
@@ -37,7 +37,7 @@ QueueItem::paintCell( QPainter *p, const QColorGroup &cg, int column, int width,
 {
     KListViewItem::paintCell( p, cg, column, width, align );
 
-    QString str = QString::number( ( (KListView *)listView() )->itemIndex( this ) + 1 );
+    QString str = QString::number( ( static_cast<KListView *>( listView() ) )->itemIndex( this ) + 1 );
 
     //draw the symbol's outline
           uint fw = p->fontMetrics().width( str ) + 2;
@@ -269,9 +269,9 @@ QueueManager::QueueManager( QWidget *parent, const char *name )
     QVBox *buttonBox = new QVBox( box );
     m_up     = new KPushButton( KGuiItem( QString::null, "up" ), buttonBox );
     m_down   = new KPushButton( KGuiItem( QString::null, "down" ), buttonBox );
-    m_remove = new KPushButton( KGuiItem( QString::null, "edit_remove" ), buttonBox );
-    m_add    = new KPushButton( KGuiItem( QString::null, "edit_add" ), buttonBox );
-    m_clear  = new KPushButton( KGuiItem( QString::null, "view_remove" ), buttonBox );
+    m_remove = new KPushButton( KGuiItem( QString::null, Amarok::icon( "dequeue_track" ) ), buttonBox );
+    m_add    = new KPushButton( KGuiItem( QString::null, Amarok::icon( "queue_track" ) ), buttonBox );
+    m_clear  = new KPushButton( KGuiItem( QString::null, Amarok::icon( "playlist_clear" ) ), buttonBox );
 
     QToolTip::add( m_up,     i18n( "Move up" ) );
     QToolTip::add( m_down,   i18n( "Move down" ) );
@@ -330,9 +330,7 @@ QueueManager::addItems( QListViewItem *after )
 
         if( current.find( item ) == current.end() ) //avoid duplication
         {
-            QString title = item->artist();
-            title.append( i18n(" - " ) );
-            title.append( item->title() );
+            QString title = i18n("%1 - %2").arg( item->artist(), item->title() );
 
             after = new QueueItem( m_listview, after, title );
             m_map[ after ] = item;
@@ -372,9 +370,7 @@ QueueManager::addQueuedItem( PlaylistItem *item )
     QValueList<PlaylistItem*>         current = m_map.values();
     QValueListIterator<PlaylistItem*> newItem = current.find( item );
 
-    QString title = item->artist();
-    title.append( i18n(" - " ) );
-    title.append( item->title() );
+    QString title = i18n("%1 - %2").arg( item->artist(), item->title() );
 
     if( newItem == current.end() ) //avoid duplication
     {
@@ -427,9 +423,7 @@ QueueManager::insertItems()
 
     for( PlaylistItem *item = list.first(); item; item = list.next() )
     {
-        QString title = item->artist();
-        title.append( i18n(" - " ) );
-        title.append( item->title() );
+        QString title = i18n("%1 - %2").arg( item->artist(), item->title() );
 
         last = new QueueItem( m_listview, last, title );
         m_map[ last ] = item;
