@@ -21,6 +21,7 @@
 #define KDE_STATUSBAR_H
 
 #include "progressBar.h" //convenience
+#include "amarok_export.h"
 #include <qwidget.h>     //baseclass
 #include <qmap.h>        //stack allocated
 #include <qvaluelist.h>  //stack allocated
@@ -35,7 +36,7 @@ namespace KIO { class Job; }
 
 namespace KDE
 {
-    class PopupMessage;
+    class OverlayWidget;
     typedef QMap<const QObject*, ProgressBar*> ProgressMap;
 
     /**
@@ -82,7 +83,7 @@ namespace KDE
     public:
         StatusBar( QWidget *parent, const char *name = "mainStatusBar" );
 
-        enum MessageType { Information, Question, Sorry, Warning, Error, ShowAgainCheckBox };
+        enum MessageType { Information, Question, Sorry, Warning, Error, ShowAgainCheckBox, None };
 
         /**
          * Start a progress operation, if owner is 0, the return value is
@@ -100,7 +101,7 @@ namespace KDE
          */
         ProgressBar &newProgressOperation( KIO::Job* );
 
-        void setProgress( const QObject *owner, int steps );
+        void incrementProgressTotalSteps( const QObject *owner, int inc = 1 );
         void incrementProgress( const QObject *owner );
         void setProgressStatus( const QObject *owner, const QString &text );
 
@@ -120,7 +121,7 @@ namespace KDE
          * Generally you should use these, as it is very easy for a user to not notice
          * statusBar messages.
          */
-        void longMessage( const QString &text, int type = Information );
+        void longMessage( const QString &text, int type = Information ) LIBAMAROK_EXPORT ;
 
         void longMessageThreadSafe( const QString &text, int type = Information );
 
@@ -129,13 +130,13 @@ namespace KDE
          * Shows a short message, with a button that can be pushed to show a long
          * message
          */
-        void shortLongMessage( const QString &_short, const QString &_long );
+        void shortLongMessage( const QString &_short, const QString &_long, int type = Information );
 
         /**
-         * Set a temporary message over the mainText label, for 2 seconds.
+         * Set a temporary message over the mainText label, for 5 seconds.
          * ONLY USE FOR STATUS MESSAGES! ie "Buffering...", "Connecting to source..."
          */
-        void shortMessage( const QString &text );
+        void shortMessage( const QString &text, bool longShort = false );
 
         /** Stop anticipating progress from sender() */
         void endProgressOperation();
@@ -149,7 +150,7 @@ namespace KDE
          * the progress bar in question
          */
         void setProgress( int steps );
-
+        void setProgress( const QObject *owner, int steps );
         /**
          * Convenience function works like setTotalSteps( QObject*, int )
          * Uses the return value from sender() to determine the owner of
@@ -174,6 +175,8 @@ namespace KDE
         void showMainProgressBar();
         void hideMainProgressBar();
         void updateProgressAppearance();
+        void showShortLongDetails();
+        void popupDeleted( QObject* );
 
     protected:
         virtual void polish();
@@ -206,17 +209,23 @@ namespace KDE
         void updateTotalProgress();
         bool allDone(); ///@return true if all progress operations are complete
         void pruneProgressBars(); /// deletes old progress bars
+        void writeLogFile( const QString &text );
 
-        QWidget *cancelButton() { return (QWidget*)child( "cancelButton" ); }
-        QWidget *toggleProgressWindowButton() { return (QWidget*)child( "showAllProgressDetails" ); }
-        QWidget *progressBox() { return (QWidget*)child( "progressBox" ); }
+        int  m_logCounter;
 
-        PopupMessage  *m_popupProgress;
+        QWidget *cancelButton() { return static_cast<QWidget*>( child( "cancelButton" ) ); }
+        QWidget *toggleProgressWindowButton() { return static_cast<QWidget*>( child( "showAllProgressDetails" ) ); }
+        QWidget *progressBox() { return static_cast<QWidget*>( child( "progressBox" ) ); }
+        QWidget *shortLongButton() { return static_cast<QWidget*>( child( "shortLongButton" ) ); }
+
+        OverlayWidget *m_popupProgress;
         QProgressBar  *m_mainProgressBar;
 
-        ProgressMap m_progressMap;
+        ProgressMap          m_progressMap;
         QValueList<QWidget*> m_messageQueue;
-        QString m_mainText;
+        QString              m_mainText;
+        QString              m_shortLongText;
+        int                  m_shortLongType;
 
         QLayout *m_otherWidgetLayout;
     };
