@@ -89,8 +89,7 @@ TagDialog::TagDialog( const MetaBundle& mb, PlaylistItem* item, QWidget* parent
 
 TagDialog::~TagDialog()
 {
-    KConfig *config = amaroK::config( "TagDialog" );
-    config->writeEntry( "CurrentTab", kTabWidget->currentPageIndex() );
+    Amarok::config( "TagDialog" )->writeEntry( "CurrentTab", kTabWidget->currentPageIndex() );
 }
 
 void
@@ -125,7 +124,7 @@ TagDialog::accept() //SLOT
 inline void
 TagDialog::openPressed() //SLOT
 {
-    amaroK::invokeBrowser( m_path );
+    Amarok::invokeBrowser( m_path );
 }
 
 
@@ -274,6 +273,8 @@ TagDialog::guessFromFilename() //SLOT
         kIntSpinBox_track->setValue( guesser.track().toInt() );
     if( !guesser.comment().isNull() )
         kTextEdit_comment->setText( guesser.comment() );
+    if( !guesser.year().isNull() )
+        kIntSpinBox_year->setValue( guesser.year().toInt() );
 }
 
 void
@@ -367,7 +368,7 @@ void TagDialog::init()
     // delete itself when closing
     setWFlags( getWFlags() | Qt::WDestructiveClose );
 
-    KConfig *config = amaroK::config( "TagDialog" );
+    KConfig *config = Amarok::config( "TagDialog" );
 
     kTabWidget->addTab( summaryTab, i18n( "Summary" ) );
     kTabWidget->addTab( tagsTab, i18n( "Tags" ) );
@@ -456,7 +457,7 @@ void TagDialog::init()
     connect( checkBox_perTrack,   SIGNAL(clicked()), SLOT(perTrack()) );
 
     // set an icon for the open-in-konqui button
-    pushButton_open->setIconSet( SmallIconSet( amaroK::icon( "files" ) ) );
+    pushButton_open->setIconSet( SmallIconSet( Amarok::icon( "files" ) ) );
 
     //Update lyrics on Context Browser
     connect( this, SIGNAL(lyricsChanged( const QString& )), ContextBrowser::instance(), SLOT( lyricsChanged( const QString& ) ) );
@@ -503,9 +504,16 @@ void TagDialog::init()
         checkBox_perTrack->hide();
 
         if( !m_playlistItem ) {
+            //We have already loaded the metadata (from the file) in the constructor
             pushButton_previous->hide();
             pushButton_next->hide();
         }
+        else
+        {
+            //Reload the metadata from the file, to be sure it's accurate
+            loadTags( m_playlistItem->url() );
+        }
+
         loadLyrics( m_bundle.url() );
         readTags();
     }
@@ -558,7 +566,7 @@ const QStringList TagDialog::statisticsData() {
         qb.addReturnValue( QueryBuilder::tabSong, QueryBuilder::valTitle );
         qb.addReturnValue( QueryBuilder::tabStats, QueryBuilder::valScore );
         qb.addMatch( QueryBuilder::tabSong, QueryBuilder::valArtistID, QString::number( artist_id ) );
-        qb.sortBy( QueryBuilder::tabStats, QueryBuilder::valPercentage, true );
+        qb.sortBy( QueryBuilder::tabStats, QueryBuilder::valForFavoriteSorting(), true );
         qb.setLimit( 0, 1 );
         values = qb.run();
         data += i18n( "Favorite by this Artist" );
@@ -570,7 +578,7 @@ const QStringList TagDialog::statisticsData() {
             qb.addReturnValue( QueryBuilder::tabSong, QueryBuilder::valTitle );
             qb.addReturnValue( QueryBuilder::tabStats, QueryBuilder::valScore );
             qb.addMatch( QueryBuilder::tabSong, QueryBuilder::valAlbumID, QString::number( album_id ) );
-            qb.sortBy( QueryBuilder::tabStats, QueryBuilder::valPercentage, true );
+            qb.sortBy( QueryBuilder::tabStats, QueryBuilder::valForFavoriteSorting(), true );
             qb.setLimit( 0, 1 );
             values = qb.run();
             data += i18n( "Favorite on this Album" );
@@ -684,6 +692,19 @@ void TagDialog::readTags()
     kIntSpinBox_score->setEnabled( local );
     kTextEdit_comment->setEnabled( local );
 
+    if( local )
+    {
+       pushButton_musicbrainz->show();
+       pushButton_guessTags->show();
+       pushButton_setFilenameSchemes->show();
+    }
+    else
+    {
+       pushButton_musicbrainz->hide();
+       pushButton_guessTags->hide();
+       pushButton_setFilenameSchemes->hide();
+    }
+
     // If it's a local file, write the directory to m_path, else disable the "open in konqui" button
     if ( local )
         m_path = m_bundle.url().directory();
@@ -1011,7 +1032,7 @@ TagDialog::bundleForURL( const KURL &url )
     if( storedTags.find( url.path() ) != storedTags.end() )
         return storedTags[ url.path() ];
 
-    return MetaBundle( url, true );
+    return MetaBundle( url, url.isLocalFile() );
 }
 
 int
@@ -1153,7 +1174,7 @@ TagDialog::writeTag( MetaBundle mb, bool updateCB )
 {
     QCString path = QFile::encodeName( mb.url().path() );
     if ( !TagLib::File::isWritable( path ) ) {
-        amaroK::StatusBar::instance()->longMessage( i18n(
+        Amarok::StatusBar::instance()->longMessage( i18n(
            "The file %1 is not writable." ).arg( path ), KDE::StatusBar::Error );
         return false;
     }
@@ -1193,7 +1214,7 @@ TagDialogWriter::doJob()
     for( int i = 0, size=m_tags.size(); i<size; ++i ) {
         QCString path = QFile::encodeName( m_tags[i].url().path() );
         if ( !TagLib::File::isWritable( path ) ) {
-            amaroK::StatusBar::instance()->longMessageThreadSafe( i18n(
+            Amarok::StatusBar::instance()->longMessageThreadSafe( i18n(
                 "The file %1 is not writable." ).arg( path ), KDE::StatusBar::Error );
             m_failed += true;
             continue;
@@ -1226,7 +1247,7 @@ TagDialogWriter::completeJob()
      if ( m_successCount )
         CollectionView::instance()->databaseChanged();
      if ( m_failCount )
-        amaroK::StatusBar::instance()->longMessage( i18n(
+        Amarok::StatusBar::instance()->longMessage( i18n(
                         "Sorry, the tag for the following files could not be changed:\n" ).arg( m_failedURLs.join( ";\n" ) ), KDE::StatusBar::Error );
 }
 
